@@ -19,8 +19,8 @@
  *
  */
 
-#include <libicq2000/SMTPClient.h>
-#include <libicq2000/exceptions.h>
+#include "SMTPClient.h"
+#include "exceptions.h"
 
 #include "sstream_fix.h"
 
@@ -31,9 +31,9 @@ using std::endl;
 namespace ICQ2000 {
 
   SMTPClient::SMTPClient(ContactRef self, const string& server_name,
-			 unsigned short server_port, Translator* translator)
-    : m_state(NOT_CONNECTED), m_recv(translator), m_server_name(server_name),
-      m_server_port(server_port), m_timeout(30), m_translator(translator),
+			 unsigned short server_port)
+    : m_state(NOT_CONNECTED), m_recv(), m_server_name(server_name),
+      m_server_port(server_port), m_timeout(30),
       m_self_contact(self)
   {
     m_socket = new TCPSocket();
@@ -209,14 +209,14 @@ namespace ICQ2000 {
   }
 
   void SMTPClient::SayHello() {
-    Buffer b(m_translator);
+    Buffer b;
     b.Pack("HELO localhost\n");
     Send(b);
     m_state = WAITING_FOR_HELO_ACK;
   }
 
   void SMTPClient::SayFrom() {
-    Buffer b(m_translator);
+    Buffer b;
 
     MessageEvent *ev = *m_msgqueue.begin();
     b.Pack("MAIL FROM:");
@@ -236,7 +236,7 @@ namespace ICQ2000 {
   }
 
   void SMTPClient::SayTo() {
-    Buffer b(m_translator);
+    Buffer b;
 
     MessageEvent *ev = *m_msgqueue.begin();
     b.Pack("RCPT TO:");
@@ -256,7 +256,7 @@ namespace ICQ2000 {
   }
 
   void SMTPClient::SayData() {
-    Buffer b(m_translator);
+    Buffer b;
 
     b.Pack("DATA\n");
     Send(b);
@@ -265,7 +265,7 @@ namespace ICQ2000 {
   }
 
   void SMTPClient::SendText() {
-    Buffer b(m_translator);
+    Buffer b;
 
     MessageEvent *ev = *m_msgqueue.begin();
 
@@ -292,7 +292,7 @@ namespace ICQ2000 {
   }
 
   void SMTPClient::SayQuit() {
-    Buffer b(m_translator);
+    Buffer b;
     b.Pack("QUIT\n");
     Send(b);
 
