@@ -1,7 +1,7 @@
 /*
 *
 * centericq core routines
-* $Id: centericq.cc,v 1.176 2003/11/21 16:51:49 konst Exp $
+* $Id: centericq.cc,v 1.185 2004/03/20 16:49:08 konst Exp $
 *
 * Copyright (C) 2001-2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -69,9 +69,11 @@ void centericq::exec() {
 	char *p = getenv("LANG");
 
 	if(p)
-	if(rus = (((string) p).substr(0, 2) == "ru"))
+	if(rus = (((string) p).substr(0, 2) == "ru")) {
+	    conf.setcharsets("cp1251", "koi8-r");
 	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
-		conf.setrussian(pname, true);
+		conf.setcpconvert(pname, true);
+	}
 
 	if(updateconf()) {
 	    manager.exec();
@@ -85,7 +87,7 @@ void centericq::exec() {
 
     if(conf.getouridcount()) {
 	conf.checkdir();
-	conf.load();
+//      conf.load();
 	groups.load();
 	clist.load();
 	lst.load();
@@ -166,7 +168,12 @@ void centericq::mainloop() {
     face.draw();
 
     while(!finished) {
-	face.status(_("F2/M current contact menu, F3/S change status, F4/G general actions, Q quit"));
+	face.status(_("%s contact menu, %s change status, %s general actions, %s quit"),
+	    face.getstatkey(key_user_menu, section_contact).c_str(),
+	    face.getstatkey(key_change_status, section_contact).c_str(),
+	    face.getstatkey(key_general_menu, section_contact).c_str(),
+	    face.getstatkey(key_quit, section_contact).c_str());
+
 	c = face.mainloop(action);
 
 	switch(action) {
@@ -185,6 +192,9 @@ void centericq::mainloop() {
 	    case ACT_ORG_GROUPS:
 		face.organizegroups();
 		break;
+	    case ACT_MASS_MOVE:
+		massmove();
+		break;
 	    case ACT_TRANSFERS:
 		face.transfermonitor();
 		break;
@@ -465,11 +475,6 @@ void centericq::joindialog() {
 	if(h.getCapabs().count(hookcapab::groupchatservices)) {
 	    if(ic.nickname.find("@") == -1)
 		ic.nickname += "@" + s.service;
-/*
-	    c->setnick(ic.nickname);
-	    c->setdispnick(c->getnick());
-	    c->setdesc(imcontact(c->getnick(), ic.pname));
-*/
 	}
 
 	if(h.getCapabs().count(hookcapab::conferencesaretemporary)) {
@@ -617,6 +622,9 @@ void centericq::checkmail() {
     static int fsize = -1;
     static int oldcount = -1;
 
+    bool lts, lo;
+    conf.getlogoptions(lts, lo);
+
     if(conf.getmailcheck())
     if(!stat(fname.c_str(), &st)) 
     if(st.st_size != fsize && (st.st_mode & S_IFREG)) {
@@ -652,6 +660,9 @@ void centericq::checkmail() {
 
 		lt = localtime(&timer_current);
 
+		if (lts)
+		face.log(_("+ new mail arrived, %d messages"), mcount);
+		else
 		face.log(_("+ [%02d:%02d:%02d] new mail arrived, %d messages"),
 		    lt->tm_hour, lt->tm_min, lt->tm_sec, mcount);
 
@@ -704,6 +715,9 @@ void centericq::checkmail() {
 		const struct tm *lt;
 		lt = localtime(&timer_current);
 
+		if (lts)
+		face.log(_("+ new mail arrived, %d messages"), mcount);
+		else
 		face.log(_("+ [%02d:%02d:%02d] new mail arrived, %d messages"),
 		    lt->tm_hour, lt->tm_min, lt->tm_sec, mcount);
 
@@ -718,7 +732,7 @@ void centericq::checkmail() {
 
 void centericq::checkconfigs() {
     static const char *configs[] = {
-	"sounds", "colorscheme", "actions", "external", 0
+	"sounds", "colorscheme", "actions", "external", "keybindings", 0
     };
 
     struct stat st;
@@ -746,6 +760,9 @@ void centericq::checkconfigs() {
 		    case 3:
 			external.load();
 			break;
+		    case 4:
+			conf.loadkeys();
+			break;
 		}
 
 		face.log(_("+ the %s configuration was changed, reloaded"), p);
@@ -1002,6 +1019,22 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
     } else if(ev.gettype() == imevent::file) {
 	sendev = new imfile(ev);
 
+    } else if(ev.gettype() == imevent::notification) {
+	const imnotification *m = dynamic_cast<const imnotification *>(&ev);
+	text = m->gettext();
+
+	if(r == icqface::reply) {
+	    if(conf.getquote()) text = quotemsg(text);
+		else text = "";
+
+	} else if(r == icqface::forward) {
+	    text = fwdnote + text;
+	}
+
+	if(!sendev) {
+	    sendev = new immessage(m->getcontact(), imevent::outgoing, text);
+	}
+
     }
 
     if(proceed = sendev) {
@@ -1266,7 +1299,7 @@ string centericq::quotemsg(const string &text) {
     return ret;
 }
 
-icqcontact *centericq::addcontact(const imcontact &ic) {
+icqcontact *centericq::addcontact(const imcontact &ic, bool reqauth) {
     icqcontact *c;
     int groupid = 0;
 
@@ -1277,6 +1310,12 @@ icqcontact *centericq::addcontact(const imcontact &ic) {
 	}
     }
 
+    imauthorization auth(ic, imevent::outgoing, imauthorization::Request, "");
+
+    if(reqauth)
+    if(!face.eventedit(auth))
+	return 0;
+
     if(conf.getgroupmode() != icqconf::nogroups) {
 	groupid = face.selectgroup(_("Select a group to add the user to"));
 	if(!groupid) return 0;
@@ -1298,15 +1337,18 @@ icqcontact *centericq::addcontact(const imcontact &ic) {
 	    }
 
 	} else {
-	    c = clist.addnew(ic, false);
+	    c = clist.addnew(ic, false, groupid, reqauth);
 	}
 
     } else {
-	c->includeintolist();
+	c->includeintolist(groupid, reqauth);
 
     }
 
-    c->setgroupid(groupid);
+    if(reqauth) {
+	em.store(auth);
+    }
+
     face.log(_("+ %s has been added to the list"), c->getdesc().totext().c_str());
     face.relaxedupdate();
 
@@ -1597,6 +1639,27 @@ void centericq::createconference(const imcontact &ic) {
     }
 }
 
+void centericq::massmove() {
+    int gid;
+    vector<imcontact>::iterator im;
+    icqcontact *c;
+
+    face.muins.clear();
+    face.multicontacts(_("Select contacts to move"));
+
+    if(!face.muins.empty())
+    if(gid = face.selectgroup(_("Mass move selected users to.."))) {
+	for(im = face.muins.begin(); im != face.muins.end(); ++im) {
+	    if(c = clist.get(*im)) {
+		c->setgroupid(gid);
+	    }
+	}
+
+	face.fillcontactlist();
+	face.update();
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 bool ischannel(const imcontact &cont) {
