@@ -7,7 +7,6 @@
 
 struct hookcapab {
     enum enumeration {
-	synclist,
 	urls,
 	files,
 	contacts,
@@ -32,7 +31,7 @@ struct hookcapab {
 	ssl,
 	channelpasswords,
 	groupchatservices,
-	nochat,
+	nochat
     };
 };
 
@@ -44,20 +43,23 @@ struct servicetype {
     };
 };
 
-struct groupoperation {
-    enum enumeration {
-	add,
-	remove,
-	rename
-    };
-};
-
 class abstracthook {
     protected:
 	enum Encoding {
 	    encUTF, encKOI, encUnknown
 	};
 
+	enum logevent {
+	    logConnecting,
+	    logLogged,
+	    logSearchFinished,
+	    logPasswordChanged,
+	    logDisconnected,
+	    logContactAdd,
+	    logContactRemove,
+	    logConfMembers
+	};
+
 	protocolname proto;
 	imstatus manualstatus;
 	verticalmenu *searchdest;
@@ -71,6 +73,27 @@ class abstracthook {
 	string getmd5(const string &text);
 
 	void requestfromfound(const imcontact &ic);
+	void log(logevent ev, ...);
+
+	struct Country_struct {
+	    char *name;
+	    unsigned short code;
+	};
+
+    public:
+	static const unsigned char Language_table_size;
+	static const char* const Language_table[];
+	static const unsigned short Country_table_size;
+	static const Country_struct Country_table[];
+
+	static string getCountryIDtoString(unsigned short id);
+	static unsigned short getCountryByName(string name);
+	static signed char getSystemTimezone();
+	static string getInterestsIDtoString(unsigned char id);
+	static string getBackgroundIDtoString(unsigned short id);
+	static string getTimezoneIDtoString(signed char id);
+	static string getTimezonetoLocaltime(signed char id);
+	static string getLanguageIDtoString(unsigned char id);
 
     public:
 	abstracthook(protocolname aproto);
@@ -114,7 +137,6 @@ class abstracthook {
 	set<hookcapab::enumeration> getCapabs() const
 	    { return fcapabs; }
 
-	virtual vector<icqcontact *> getneedsync();
 	virtual void ouridchanged(const icqconf::imaccount &ia);
 
 	virtual bool knowntransfer(const imfile &fr) const;
@@ -132,6 +154,9 @@ class abstracthook {
 
 	virtual void updatecontact(icqcontact *c);
 	virtual void renamegroup(const string &oldname, const string &newname);
+
+	virtual bool regconnect(const string &aserv);
+	virtual bool regattempt(unsigned int &auin, const string &apassword);
 };
 
 abstracthook &gethook(protocolname pname);
