@@ -1,7 +1,7 @@
 /*
 *
 * centericq AIM protocol handling class
-* $Id: aimhook.cc,v 1.36 2003/10/11 14:28:12 konst Exp $
+* $Id: aimhook.cc,v 1.40 2004/01/27 00:43:30 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -47,7 +47,6 @@ aimhook::aimhook()
     fcapabs.insert(hookcapab::setaway);
     fcapabs.insert(hookcapab::changepassword);
     fcapabs.insert(hookcapab::changedetails);
-    fcapabs.insert(hookcapab::synclist);
     fcapabs.insert(hookcapab::directadd);
     fcapabs.insert(hookcapab::changeabout);
 }
@@ -61,7 +60,7 @@ void aimhook::init() {
 
 void aimhook::connect() {
     icqconf::imaccount acc = conf.getourid(aim);
-    face.log(_("+ [aim] connecting to the server"));
+    log(logConnecting);
 
     if(handle) {
 	firetalk_register_callback(handle, FC_DISCONNECT, 0);
@@ -192,7 +191,7 @@ bool aimhook::send(const imevent &ev) {
 void aimhook::sendnewuser(const imcontact &ic) {
     if(logged())
     if(find(buddies.begin(), buddies.end(), ic.nickname) == buddies.end()) {
-	face.log(_("+ [aim] adding %s to the contacts"), ic.nickname.c_str());
+	log(logContactAdd, ic.nickname.c_str());
 	firetalk_im_add_buddy(handle, ic.nickname.c_str());
 	firetalk_save_config(handle);
 	requestinfo(ic);
@@ -205,11 +204,7 @@ void aimhook::removeuser(const imcontact &ic) {
 
 void aimhook::removeuser(const imcontact &ic, bool report) {
     if(online()) {
-	if(report) {
-	    face.log(_("+ [aim] removing %s from the contacts"),
-		ic.nickname.c_str());
-	}
-
+	if(report) log(logContactRemove, ic.nickname.c_str());
 	firetalk_im_remove_buddy(handle, ic.nickname.c_str());
 	firetalk_save_config(handle);
     }
@@ -334,29 +329,11 @@ void aimhook::resolve() {
     }
 }
 
-vector<icqcontact *> aimhook::getneedsync() {
-    vector<icqcontact *> r;
-
-    buddies.clear();
-    firetalk_im_list_buddies(handle);
-
-    for(int i = 0; i < clist.count; i++) {
-	icqcontact *c = (icqcontact *) clist.at(i);
-
-	if(c->getdesc().pname == aim) {
-	    if(find(buddies.begin(), buddies.end(), c->getdesc().nickname) == buddies.end())
-		r.push_back(c);
-	}
-    }
-
-    return r;
-}
-
 // ----------------------------------------------------------------------------
 
 void aimhook::connected(void *connection, void *cli, ...) {
     ahook.flogged = true;
-    face.log(_("+ [aim] logged in"));
+    ahook.log(logLogged);
     face.update();
 
     ahook.ourstatus = available;
@@ -374,8 +351,7 @@ void aimhook::disconnected(void *connection, void *cli, ...) {
     ahook.fonline = false;
     logger.putourstatus(aim, ahook.getstatus(), offline);
     clist.setoffline(aim);
-
-    face.log(_("+ [aim] disconnected"));
+    ahook.log(logDisconnected);
     face.update();
 }
 
@@ -411,8 +387,7 @@ void aimhook::newpass(void *connection, void *cli, ...) {
 	icqconf::imaccount acc = conf.getourid(aim);
 	acc.password = pass;
 	conf.setourid(acc);
-
-	face.log(_("+ [aim] password was changed successfully"));
+	ahook.log(logPasswordChanged);
     }
 
     DLOG("newpass");
@@ -433,7 +408,7 @@ void aimhook::gotinfo(void *conn, void *cli, ...) {
 	icqcontact *c = clist.get(imcontact(nick, aim));
 
 	if(c) {
-	    c->setabout(cuthtml(info, true));
+	    c->setabout(cuthtml(info, chCutBR | chLeaveLinks));
 	    if(c->getabout().empty())
 		c->setabout(_("The user has no profile information."));
 	}
@@ -454,7 +429,7 @@ void aimhook::getmessage(void *conn, void *cli, ...) {
     if(sender && message)
     if(strlen(sender) && strlen(message)) {
 	em.store(immessage(imcontact(sender, aim),
-	    imevent::incoming, ahook.rushtmlconv("wk", cuthtml(message, true))));
+	    imevent::incoming, ahook.rushtmlconv("wk", cuthtml(message, chCutBR | chLeaveLinks))));
     }
 
     DLOG("getmessage");
