@@ -1,9 +1,9 @@
 /*
 *
 * centericq icq protocol handling class
-* $Id: icqhook.cc,v 1.138 2003/10/15 23:40:19 konst Exp $
+* $Id: icqhook.cc,v 1.148 2004/03/09 21:46:26 konst Exp $
 *
-* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -26,16 +26,18 @@
 #include "icqconf.h"
 #include "icqface.h"
 #include "imlogger.h"
+#include "icqgroups.h"
 
 #include "accountmanager.h"
 #include "eventmanager.h"
 
-#include <libicq2000/userinfohelpers.h>
-
 #define PERIOD_ICQPOLL  5
 #define PERIOD_RESOLVE  10
 #define DELAY_SENDNEW   5
 
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
 icqhook ihook;
 
 static const Status stat2int[imstatus_size] = {
@@ -57,9 +59,8 @@ icqhook::icqhook(): abstracthook(icq) {
     fcapabs.insert(hookcapab::setaway);
     fcapabs.insert(hookcapab::fetchaway);
     fcapabs.insert(hookcapab::changenick);
-    fcapabs.insert(hookcapab::changepassword);
+//    fcapabs.insert(hookcapab::changepassword);
     fcapabs.insert(hookcapab::changedetails);
-    fcapabs.insert(hookcapab::synclist);
     fcapabs.insert(hookcapab::authrequests);
     fcapabs.insert(hookcapab::authreqwithmessages);
     fcapabs.insert(hookcapab::contacts);
@@ -67,8 +68,6 @@ icqhook::icqhook(): abstracthook(icq) {
     fcapabs.insert(hookcapab::cltemporary);
     fcapabs.insert(hookcapab::changeabout);
 
-    cli.setServerSideGroup("centericq", 0x0666);
-
     cli.connected.connect(this, &icqhook::connected_cb);
     cli.disconnected.connect(this, &icqhook::disconnected_cb);
     cli.socket.connect(this, &icqhook::socket_cb);
@@ -79,12 +78,12 @@ icqhook::icqhook(): abstracthook(icq) {
     cli.contact_status_change_signal.connect(this, &icqhook::contact_status_change_signal_cb);
     cli.newuin.connect(this, &icqhook::newuin_cb);
     cli.rate.connect(this, &icqhook::rate_cb);
-    cli.password_changed.connect(this, &icqhook::password_changed_cb);
+//    cli.password_changed.connect(this, &icqhook::password_changed_cb);
     cli.want_auto_resp.connect(this, &icqhook::want_auto_resp_cb);
     cli.search_result.connect(this, &icqhook::search_result_cb);
-    cli.server_based_contact_list.connect(this, &icqhook::server_based_contact_list_cb);
     cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
     cli.self_contact_status_change_signal.connect(this, &icqhook::self_contact_status_change_cb);
+    cli.sbl_received.connect(this, &icqhook::sbl_received_cb);
 
 #ifdef DEBUG
     cli.logger.connect(this, &icqhook::logger_cb);
@@ -107,8 +106,7 @@ icqhook::~icqhook() {
     cli.want_auto_resp.disconnect(this);
     cli.self_contact_userinfo_change_signal.disconnect(this);
     cli.self_contact_status_change_signal.disconnect(this);
-    cli.server_based_contact_list.disconnect(this);
-    cli.password_changed.disconnect(this);
+//    cli.password_changed.disconnect(this);
 }
 
 void icqhook::init() {
@@ -120,19 +118,9 @@ void icqhook::connect() {
     int i, ptpmin, ptpmax;
     icqcontact *c;
 
-    if(acc.additional["autosync"].empty()) acc.additional["autosync"] = "1";
     if(acc.additional["webaware"].empty()) acc.additional["webaware"] = "1";
     conf.setourid(acc);
 
-    for(i = 0; i < clist.count; i++) {
-	c = (icqcontact *) clist.at(i);
-
-	if(c->getdesc().pname == proto && c->getdesc().uin) {
-	    ContactRef ct(new Contact(c->getdesc().uin));
-	    cli.addContact(ct);
-	}
-    }
-
     if(!acc.server.empty()) cli.setLoginServerHost(acc.server);
     if(acc.port) cli.setLoginServerPort(acc.port);
 
@@ -151,7 +139,7 @@ void icqhook::connect() {
 	cli.setAcceptInDC(false);
     }
 
-    face.log(_("+ [icq] connecting to the server"));
+    log(logConnecting);
 
     cli.self_contact_userinfo_change_signal.disconnect(this);
 
@@ -159,11 +147,23 @@ void icqhook::connect() {
     cli.setPassword(acc.password);
     cli.setWebAware(acc.additional["webaware"] == "1");
 
-    sendinvisible();
     cli.setStatus(stat2int[manualstatus], manualstatus == invisible);
-
     cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
 
+    for(i = 0; i < clist.count; i++) {
+	c = (icqcontact *) clist.at(i);
+
+	if(c->getdesc().pname == proto && c->getdesc().uin) {
+	    icqcontact::basicinfo bi = c->getbasicinfo();
+	    bi.authawait = true;
+	    c->setbasicinfo(bi);
+
+	    addContact(c->getdesc().uin, groups.getname(c->getgroupid()));
+	}
+    }
+
+    sendinvisible();
+
     fonline = true;
     flogged = false;
 }
@@ -426,7 +426,10 @@ bool icqhook::send(const imevent &ev) {
 	    } else {
 		ic = new Contact(m->getphone());
 		ic->setMobileNo(m->getphone());
-		cli.addContact(ic);
+
+		ContactTree &ct = cli.getContactTree();
+		ContactTree::Group &g = ct.group_size() ? *ct.begin() : ct.add_group("New");
+		g.add(ic);
 	    }
 
 	} else {
@@ -435,7 +438,7 @@ bool icqhook::send(const imevent &ev) {
 	    cli.contact_userinfo_change_signal.connect(this, &icqhook::contact_userinfo_change_signal_cb);
 	}
 
-	sev = new SMSMessageEvent(ic, siconv(m->getmessage(), conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset(), "utf-8"), true);
+	sev = new SMSMessageEvent(ic, rusconv("ku", m->getmessage()), true);
 
     } else if(ev.gettype() == imevent::authorization) {
 	const imauthorization *m = static_cast<const imauthorization *> (&ev);
@@ -487,43 +490,52 @@ bool icqhook::send(const imevent &ev) {
 
 void icqhook::sendnewuser(const imcontact &ic) {
     static time_t lastadd = 0;
+    icqcontact *cc = clist.get(ic);
 
-    if(logged() && ic.uin) {
+    if(logged() && ic.uin && cc) {
 	if(timer_current-lastadd > DELAY_SENDNEW) {
-	    cli.addContact(new Contact(ic.uin));
-	    cli.fetchSimpleContactInfo(cli.getContact(ic.uin));
-	    cli.fetchDetailContactInfo(cli.getContact(ic.uin));
+	    ContactRef cont = addContact(ic.uin, groups.getname(cc->getgroupid()));
+	    cli.fetchSimpleContactInfo(cont);
+	    cli.fetchDetailContactInfo(cont);
 
-	    if(conf.getourid(icq).additional["autosync"] == "1") {
-		icqcontact *cc = clist.get(ic);
-		if(cc)
-		if(cc->inlist())
-		    cli.uploadServerBasedContactList(cli.getContact(ic.uin));
+	    if(sblrecv && cc->inlist()) {
+		ContactTree::Group &g = cli.getContactTree().lookup_group_containing_contact(cont);
+
+		cont->setServerSideInfo(g.get_id(), 0);
+		cont->setServerBased(true);
+		cont->setAuthAwait(cc->getbasicinfo().authawait);
+
+		cli.uploadServerBasedContact(cont);
 	    }
 
 	    lastadd = timer_current;
+
 	} else {
 	    uinstosend.push_back(ic);
+
 	}
     }
 }
 
 void icqhook::removeuser(const imcontact &c) {
-    if(conf.getourid(icq).additional["autosync"] == "1")
-    if(cli.getContact(c.uin).get()) {
+    ContactRef ic = cli.getContact(c.uin);
+
+    if(ic.get()) {
 	icqcontact *cc = clist.get(c);
+
 	if(cc)
 	if(cc->inlist()) {
-	    ContactRef ic = cli.getContact(c.uin);
 	    if(ic->getServerBased()) {
 		ic->setAlias(cc->getnick());
 		ic->setAuthAwait(cc->getbasicinfo().authawait);
-		cli.removeServerBasedContactList(ic);
+		cli.removeServerBasedContact(ic);
 	    }
 	}
-    }
 
-    cli.removeContact(c.uin);
+	ContactTree& ct = cli.getContactTree();
+	ContactTree::Group &gr = ct.lookup_group_containing_contact(ic);
+	gr.remove(c.uin);
+    }
 }
 
 void icqhook::setautostatus(imstatus st) {
@@ -626,7 +638,7 @@ void icqhook::requestinfo(const imcontact &c) {
 	    ContactRef icont = cli.getContact(c.uin);
 
 	    if(!icont.get()) {
-		cli.addContact(new Contact(c.uin));
+		addContact(c.uin, "");
 	    } else {
 		cli.fetchSimpleContactInfo(icont);
 		cli.fetchDetailContactInfo(icont);
@@ -647,7 +659,7 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 
     sex = params.gender == genderMale ? ICQ2000::SEX_MALE :
 	params.gender == genderFemale ? ICQ2000::SEX_FEMALE :
-	ICQ2000::SEX_UNSPECIFIED;
+	ICQ2000::SEX_UNKNOWN;
 
     if(params.uin) {
 	searchevent = cli.searchForContacts(params.uin);
@@ -663,7 +675,7 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
     !params.company.empty() || !params.department.empty() ||
     !params.position.empty() || params.onlineonly ||
     params.country || params.language ||
-    (sex != ICQ2000::SEX_UNSPECIFIED) || (params.agerange != range_NoRange)) {
+    (sex != SEX_UNKNOWN) || (params.agerange != RANGE_NORANGE)) {
 	searchevent = cli.searchForContacts(rusconv("kw", params.nick),
 	    rusconv("kw", params.firstname), rusconv("kw", params.lastname),
 	    rusconv("kw", params.email), params.agerange, sex,
@@ -710,8 +722,8 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     home.setMobileNo(rusconv("kw", cbinfo.cellular));
 
     home.zip = cbinfo.zip;
-    home.country = cbinfo.country;
-    home.timezone = UserInfoHelpers::getSystemTimezone();
+    home.country = (Country) cbinfo.country;
+    home.timezone = (Timezone) getSystemTimezone();
 
     /* more information */
 
@@ -720,16 +732,16 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     hpage.sex =
 	cminfo.gender == genderFemale ? SEX_FEMALE :
 	cminfo.gender == genderMale ? SEX_MALE :
-	    SEX_UNSPECIFIED;
+	    SEX_UNKNOWN;
 
     hpage.homepage = rusconv("kw", cminfo.homepage);
     hpage.birth_day = cminfo.birth_day;
     hpage.birth_month = cminfo.birth_month;
     hpage.birth_year = cminfo.birth_year;
 
-    hpage.lang1 = cminfo.lang1;
-    hpage.lang2 = cminfo.lang2;
-    hpage.lang3 = cminfo.lang3;
+    hpage.lang1 = (Language) cminfo.lang1;
+    hpage.lang2 = (Language) cminfo.lang2;
+    hpage.lang3 = (Language) cminfo.lang3;
 
     /* work information */
 
@@ -752,7 +764,6 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     icqconf::imaccount acc = conf.getourid(icq);
     acc.additional["webaware"] = cbinfo.webaware ? "1" : "0";
     acc.additional["randomgroup"] = i2str(cbinfo.randomgroup);
-    acc.additional["autosync"] = cbinfo.autosync ? "1" : "0";
     conf.setourid(acc);
 
     cli.setWebAware(cbinfo.webaware);
@@ -760,9 +771,9 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
 
     cli.uploadSelfDetails();
     cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
-
+/*
     if(!c.getreginfo().password.empty())
-	cli.changePassword(c.getreginfo().password);
+	cli.changePassword(c.getreginfo().password);*/
 }
 
 void icqhook::requestawaymsg(const imcontact &c) {
@@ -798,7 +809,7 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 
 	cbinfo.fname = rusconv("wk", ic->getFirstName());
 	cbinfo.lname = rusconv("wk", ic->getLastName());
-	cbinfo.email = rusconv("wk", ic->getEmail());
+	if(!ic->getEmail().empty()) cbinfo.email = rusconv("wk", ic->getEmail());
 	cbinfo.city = rusconv("wk", home.city);
 	cbinfo.state = rusconv("wk", home.state);
 	cbinfo.phone = rusconv("wk", home.phone);
@@ -848,7 +859,7 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 	/* personal interests */
 
 	for(ii = pint.interests.begin(); ii != pint.interests.end(); ++ii) {
-	    sbuf = UserInfoHelpers::getInterestsIDtoString(ii->first);
+	    sbuf = getInterestsIDtoString(ii->first);
 
 	    if(!ii->second.empty()) {
 		if(!sbuf.empty()) sbuf += ": ";
@@ -861,7 +872,7 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 	/* education background */
 
 	for(isc = backg.schools.begin(); isc != backg.schools.end(); ++isc) {
-	    sbuf = UserInfoHelpers::getBackgroundIDtoString(isc->first);
+	    sbuf = getBackgroundIDtoString(isc->first);
 	    if(!sbuf.empty()) sbuf += ": ";
 	    sbuf += rusconv("wk", isc->second);
 	    if(!sbuf.empty()) backginfo.push_back(sbuf);
@@ -922,91 +933,60 @@ void icqhook::processemailevent(const string &sender, const string &email, const
     }
 }
 
-void icqhook::synclist() {
-    fd_set srfds, swfds, sefds;
-    struct timeval tv;
-    int hsockfd, s;
-    time_t t = time(0);
+void icqhook::updatecontact(icqcontact *c) {
+    if(sblrecv) {
+	ContactTree &tree = cli.getContactTree();
+	ContactRef ic = tree[c->getdesc().uin];
 
-    blockmode = SyncList;
-    syncstatus = reqUpload;
+	if(ic.get()) {
+	    string gname = groups.getname(c->getgroupid());
+	    ContactTree::Group &oldg = tree.lookup_group_containing_contact(ic);
+	    ContactTree::Group *newg = 0;
 
-    /*
-    *
-    * Let's now make a list of contacts to be uploaded.
-    *
-    */
+	    if(oldg.get_label() != gname) {
+		ContactTree::iterator curr = tree.begin();
+		while(curr != tree.end()) {
+		    if((*curr).get_label() == gname) {
+			newg = &(*curr);
+			break;
+		    }
+		    ++curr;
+		}
 
-    ContactList tcl;
-    vector<icqcontact *> tobestored;
-    vector<icqcontact *>::iterator ic;
+		cli.removeServerBasedContact(ic);
 
-    cli.contact_userinfo_change_signal.disconnect(this);
-    getsyncstatus(s, tobestored);
+		if(!newg) newg = &(tree.add_group(gname));
+		tree.relocate_contact(ic, oldg, *newg);
 
-    for(ic = tobestored.begin(); ic != tobestored.end() && tcl.size() <= 10; ++ic) {
-	ContactRef ct = cli.getContact((*ic)->getdesc().uin);
-	if(ct.get()) {
-	    ct->setAlias((*ic)->getnick());
-	    tcl.add(ct);
-	}
-    }
+		ic->setServerSideInfo(newg->get_id(), ic->getServerSideID());
+		ic->setAuthAwait(c->getbasicinfo().authawait);
 
-    cli.contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
-    cli.uploadServerBasedContactList(tcl);
-
-    while(syncstatus != ackFetch && logged() && time(0)-t <= 30) {
-	hsockfd = 0;
-
-	FD_ZERO(&srfds);
-	FD_ZERO(&swfds);
-	FD_ZERO(&sefds);
-
-	getsockets(srfds, swfds, sefds, hsockfd);
-
-	tv.tv_sec = 10;
-	tv.tv_usec = 0;
-
-	select(hsockfd+1, &srfds, &swfds, &sefds, &tv);
-
-	if(isoursocket(srfds, swfds, sefds)) {
-	    main();
-
-	    switch(syncstatus) {
-		case ackUpload:
-		    syncstatus = reqFetch;
-		    cli.fetchServerBasedContactList();
-		    break;
+		cli.uploadServerBasedContact(ic);
 	    }
 	}
     }
-
-    blockmode = Normal;
 }
 
-void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored) {
-    ContactList &cl = cli.getContactList();
-    ContactList::iterator ic = cl.begin();
-
-    tobestored.clear();
-    synchronized = 0;
-
-    while(ic != cl.end()) {
-	imcontact imc = imcontact((*ic)->getUIN(), icq);
-	icqcontact *c = clist.get(imc);
-
-	if(c)
-	if(c->inlist()) {
-	    if(!(*ic)->getServerBased()) {
-		tobestored.push_back(c);
-
-	    } else {
-		synchronized++;
+void icqhook::renamegroup(const string &oldname, const string &newname) {
+    if(logged()) {
+/*
+	ContactTree::Group *gp = 0;
+	ContactTree &ct = cli.getContactTree();
 
+	ContactTree::iterator curr = ct.begin();
+	while(curr != ct.end()) {
+	    if((*curr).get_label() == oldname) {
+		gp = &(*curr);
+		break;
 	    }
+	    ++curr;
 	}
 
-	++ic;
+	if(gp) {
+	    gp->set_label(newname);
+	    cli.uploadServerBasedGroup(*gp);
+	}
+*/
     }
 }
 
@@ -1022,7 +1002,7 @@ void icqhook::connected_cb(ConnectedEvent *ev) {
 
     cli.setRandomChatGroup(strtoul(conf.getourid(icq).additional["randomgroup"].c_str(), 0, 0));
 
-    face.log(_("+ [icq] logged in"));
+    log(logLogged);
     face.update();
 
     string buf;
@@ -1035,14 +1015,13 @@ void icqhook::connected_cb(ConnectedEvent *ev) {
 	getstring(f, buf); cli.getSelfContact()->setLastName(buf);
 	f.close();
 
-	cli.getSelfContact()->getMainHomeInfo().timezone = UserInfoHelpers::getSystemTimezone();
+	cli.getSelfContact()->getMainHomeInfo().timezone = (Timezone) getSystemTimezone();
 	cli.uploadSelfDetails();
 
 	unlink(conf.getconfigfname("icq-infoset").c_str());
     }
 
-    if(conf.getourid(icq).additional["autosync"] == "1")
-	cli.fetchServerBasedContactList();
+    cli.fetchServerBasedContactList();
 }
 
 void icqhook::disconnected_cb(DisconnectedEvent *ev) {
@@ -1090,7 +1069,7 @@ void icqhook::messaged_cb(MessageEvent *ev) {
 
     if(ev->getType() == MessageEvent::Normal) {
 	NormalMessageEvent *r = static_cast<NormalMessageEvent *>(ev);
-	em.store(immessage(ic, imevent::incoming, rusconv("wk", r->getMessage())));
+	em.store(immessage(ic, imevent::incoming, rusconv("wk", r->getMessage()), r->getTime()));
 
     } else if(ev->getType() == MessageEvent::URL) {
 	URLMessageEvent *r = static_cast<URLMessageEvent *>(ev);
@@ -1113,8 +1092,7 @@ void icqhook::messaged_cb(MessageEvent *ev) {
 	}
 
 	if(c) {
-	    em.store(imsms(c, imevent::incoming, siconv(r->getMessage(),
-		"utf-8", conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset())));
+	    em.store(imsms(c, imevent::incoming, rusconv("uk", r->getMessage())));
 	}
 
     } else if(ev->getType() == MessageEvent::AuthReq) {
@@ -1130,6 +1108,13 @@ void icqhook::messaged_cb(MessageEvent *ev) {
 	    icqcontact::basicinfo bi = c->getbasicinfo();
 	    bi.authawait = false;
 	    c->setbasicinfo(bi);
+
+	    ContactRef cr = cli.getContactTree()[ic.uin];
+	    if(cr.get()) {
+		cr->setAuthAwait(false);
+		cli.removeServerBasedContact(cr);
+		cli.uploadServerBasedContact(cr);
+	    }
 	}
 
 	if(r->isGranted()) {
@@ -1340,10 +1325,7 @@ void icqhook::want_auto_resp_cb(ICQMessageEvent *ev) {
 
     ident = cont.totext();
     if(c) ident += " (" + c->getdispnick() + ")";
-/*
-    face.log(_("+ [icq] %s has requested our away message"), ident.c_str());
-    sprintf(buf, _("%s requested our away message, sent the response"), ident.c_str());
-*/
+
     logger.putmessage(buf);
     ev->setAwayMessage(rusconv("kw", conf.getawaymsg(icq)));
 }
@@ -1377,6 +1359,9 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 	    line += " " + binfo.fname + " " + binfo.lname;
 	    if(!binfo.email.empty()) line += " <" + binfo.email + ">";
 
+	    binfo.requiresauth = cont->getAuthReq();
+	    binfo.authawait = cont->getAuthAwait();
+
 	    c->setbasicinfo(binfo);
 
 	    foundguys.push_back(c);
@@ -1386,10 +1371,7 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 
 	if(ev->isFinished() && searchdest) {
 	    face.findready();
-
-	    face.log(_("+ [icq] search finished, %d found"),
-		ev->getContactList().size());
-
+	    log(logSearchFinished, ev->getContactList().size());
 	    searchdest = 0;
 	}
     }
@@ -1412,7 +1394,6 @@ void icqhook::self_contact_userinfo_change_cb(UserInfoChangeEvent *ev) {
 	icqcontact::basicinfo cbinfo = c->getbasicinfo();
 	cbinfo.webaware = im.additional["webaware"] == "1";
 	cbinfo.randomgroup = strtoul(im.additional["randomgroup"].c_str(), 0, 0);
-	cbinfo.autosync = im.additional["autosync"] == "1";
 	c->setbasicinfo(cbinfo);
     }
 }
@@ -1421,34 +1402,92 @@ void icqhook::self_contact_status_change_cb(StatusChangeEvent *ev) {
     face.update();
 }
 
+/*
+
 void icqhook::password_changed_cb(PasswordChangeEvent *ev) {
     if(ev->isSuccess()) {
 	icqconf::imaccount acc = conf.getourid(icq);
 	acc.password = ev->getPassword();
 	conf.setourid(acc);
-
-	face.log(_("+ [icq] password was changed successfully"));
+	log(logPasswordChanged);
     }
 }
 
-void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
-    const ContactList &lst = ev->getContactList();
-    ContactList::const_iterator i = lst.begin();
-    icqcontact *c;
+*/
 
-    if(ev->getType() == ServerBasedContactEvent::Fetch) {
-	syncstatus = ackFetch;
+void icqhook::sbl_received_cb(SBLReceivedEvent *ev) {
+    icqcontact *cc;
+    ContactTree &tree = cli.getContactTree();
+    ContactTree::iterator curr = ev->tree.begin();
 
-	while(i != lst.end()) {
-	    imcontact cont((*i)->getUIN(), icq);
-	    if(!clist.get(cont)) clist.addnew(cont, false);
-	    ++i;
+    sblrecv = false;
+
+    while(curr != ev->tree.end()) {
+	ContactTree::Group *lg = 0;
+
+	ContactTree::iterator igl = tree.begin();
+	while(igl != tree.end()) {
+	    if((*igl).get_label() == (*curr).get_label()) {
+		lg = &(*igl);
+		break;
+	    }
+	    ++igl;
 	}
 
-    } else if(ev->getType() == ServerBasedContactEvent::Upload) {
-	syncstatus = ackUpload;
+	if(!lg) {
+	    lg = &(tree.add_group((*curr).get_label(), (*curr).get_id()));
+	} else {
+	    lg->set_id((*curr).get_id());
+	}
 
-    } else if(ev->getType() == ServerBasedContactEvent::Remove) {
+	ContactTree::Group::iterator ig = curr->begin();
+	while(ig != curr->end()) {
+	    imcontact ic((*ig)->getUIN(), proto);
+	    clist.updateEntry(ic, (*curr).get_label());
 
+	    ContactRef nc = tree[ic.uin];
+	    if(nc.get()) {
+		ContactTree::Group &cg = tree.lookup_group_containing_contact(nc);
+		tree.relocate_contact(nc, cg, *lg);
+	    } else {
+		nc = lg->add(new Contact(ic.uin));
+		nc->setAuthAwait((*ig)->getAuthAwait());
+	    }
+
+	    if(cc = clist.get(ic)) {
+		icqcontact::basicinfo bi = cc->getbasicinfo();
+		bi.authawait = nc->getAuthAwait();
+		cc->setbasicinfo(bi);
+	    }
+
+	    nc->setServerBased(true);
+	    nc->setServerSideInfo((*ig)->getServerSideGroupID(), (*ig)->getServerSideID());
+
+	    ++ig;
+	}
+	++curr;
     }
+
+    sblrecv = true;
+}
+
+ContactRef icqhook::addContact(unsigned int uin, const string &groupname) {
+    ContactTree& ct = cli.getContactTree();
+    ContactTree::Group *gp = 0;
+
+    ContactTree::iterator curr = ct.begin();
+    while(curr != ct.end()) {
+	if((*curr).get_label() == groupname) {
+	    gp = &(*curr);
+	    break;
+	}
+	++curr;
+    }
+
+    if(!gp) gp = &(ct.add_group(groupname));
+
+    ContactRef cont(new Contact(uin));
+    gp->add(cont);
+
+    return cont;
 }
