@@ -1,9 +1,9 @@
 /*
 *
 * centericq Jabber protocol handling class
-* $Id: jabberhook.cc,v 1.60 2003/11/22 19:14:33 konst Exp $
+* $Id: jabberhook.cc,v 1.69 2004/02/20 20:48:47 konst Exp $
 *
-* Copyright (C) 2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2002-2005 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,6 @@
 #include "eventmanager.h"
 #include "icqgroups.h"
 
-#include <libicq2000/userinfohelpers.h>
-
 #define DEFAULT_CONFSERV "conference.jabber.org"
 
 static void jidsplit(const string &jid, string &user, string &host, string &rest) {
@@ -54,16 +52,15 @@ static void jidsplit(const string &jid, string &user, string &host, string &rest
 static string jidtodisp(const string &jid) {
     string user, host, rest;
     jidsplit(jid, user, host, rest);
-
+/*
     if(host != "jabber.com" && !host.empty())
 	user += (string) "@" + host;
+*/
+    user += (string) "@" + host;
 
     return user;
 }
 
-#define KOI2UTF(x) siconv(x, conf.getrussian(jabber) ? "koi8-u" : conf.getdefcharset(), "utf-8")
-#define UTF2KOI(x) siconv(x, "utf-8", conf.getrussian(jabber) ? "koi8-u" : conf.getdefcharset())
-
 // ----------------------------------------------------------------------------
 
 jabberhook jhook;
@@ -94,7 +91,6 @@ jabberhook::jabberhook(): abstracthook(jabber), jc(0), flogged(false), fonline(f
     fcapabs.insert(hookcapab::groupchatservices);
     fcapabs.insert(hookcapab::changenick);
     fcapabs.insert(hookcapab::changeabout);
-    fcapabs.insert(hookcapab::synclist);
     fcapabs.insert(hookcapab::version);
 }
 
@@ -109,7 +105,7 @@ void jabberhook::connect() {
     icqconf::imaccount acc = conf.getourid(proto);
     string jid = getourjid();
 
-    face.log(_("+ [jab] connecting to the server"));
+    log(logConnecting);
 
     auto_ptr<char> cjid(strdup(jid.c_str()));
     auto_ptr<char> cpass(strdup(acc.password.c_str()));
@@ -242,7 +238,7 @@ bool jabberhook::send(const imevent &ev) {
 	    return true;
 	}
 
-	text = KOI2UTF(text);
+	text = rusconv("ku", text);
 
 	auto_ptr<char> cjid(strdup(jidnormalize(c->getdesc().nickname).c_str()));
 	auto_ptr<char> ctext(strdup(text.c_str()));
@@ -283,11 +279,11 @@ void jabberhook::sendnewuser(const imcontact &ic, bool report) {
 
     if(!ischannel(ic)) {
 	auto_ptr<char> cjid(strdup(jidnormalize(ic.nickname).c_str()));
-	if(roster.find(cjid.get()) != roster.end())
+	if(roster.find(cjid.get()) != roster.end()) {
 	    return;
+	}
 
-	if(report)
-	    face.log(_("+ [jab] adding %s to the contacts"), ic.nickname.c_str());
+	if(report) log(logContactAdd, ic.nickname.c_str());
 
 	x = jutil_presnew(JPACKET__SUBSCRIBE, cjid.get(), 0);
 	jab_send(jc, x);
@@ -316,9 +312,13 @@ void jabberhook::sendnewuser(const imcontact &ic, bool report) {
 	    imcontact icc(jidtodisp(ic.nickname), proto);
 	    if(ic.nickname != icc.nickname) {
 		c->setdesc(icc);
-		c->setdispnick(icc.nickname);
 	    }
+
 	    c->setnick("");
+
+	    string u, h, r;
+	    jidsplit(icc.nickname, u, h, r);
+	    c->setdispnick(u);
 	}
 
 	requestinfo(c);
@@ -371,19 +371,19 @@ void jabberhook::removeuser(const imcontact &ic, bool report) {
 
 	}
 
-	    if(report) face.log(_("+ [jab] removing %s from the contacts"), ic.nickname.c_str());
+	if(report) log(logContactRemove, ic.nickname.c_str());
 
-	    x = jutil_presnew(JPACKET__UNSUBSCRIBE, cjid.get(), 0);
-	    jab_send(jc, x);
-	    xmlnode_free(x);
+	x = jutil_presnew(JPACKET__UNSUBSCRIBE, cjid.get(), 0);
+	jab_send(jc, x);
+	xmlnode_free(x);
 
-	    x = jutil_iqnew(JPACKET__SET, NS_ROSTER);
-	    y = xmlnode_get_tag(x, "query");
-	    z = xmlnode_insert_tag(y, "item");
-	    xmlnode_put_attrib(z, "jid", cjid.get());
-	    xmlnode_put_attrib(z, "subscription", "remove");
-	    jab_send(jc, x);
-	    xmlnode_free(x);
+	x = jutil_iqnew(JPACKET__SET, NS_ROSTER);
+	y = xmlnode_get_tag(x, "query");
+	z = xmlnode_insert_tag(y, "item");
+	xmlnode_put_attrib(z, "jid", cjid.get());
+	xmlnode_put_attrib(z, "subscription", "remove");
+	jab_send(jc, x);
+	xmlnode_free(x);
 
     } else {
 	if(c = clist.get(ic)) {
@@ -555,7 +555,7 @@ void jabberhook::setjabberstatus(imstatus st, const string &msg) {
 
     if(!msg.empty())
 	xmlnode_insert_cdata(xmlnode_insert_tag(x, "status"),
-	    msg.c_str(), (unsigned) -1);
+	    rusconv("ku", msg).c_str(), (unsigned) -1);
 
     jab_send(jc, x);
     xmlnode_free(x);
@@ -605,7 +605,7 @@ vector<string> jabberhook::getservices(servicetype::enumeration st) const {
 	    break;
 	case servicetype::groupchat:
 	    while(ia != agents.end()) {
-		if(ia->type == agent::atGroupchat) r.push_back(ia->name);
+		if(ia->type == agent::atGroupchat) r.push_back(ia->jid);
 		++ia;
 	    }
 	default:
@@ -733,8 +733,7 @@ void jabberhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	}
 
 	face.findready();
-	face.log(_("+ [jab] conference list fetching finished, %d found"),
-	    foundguys.size());
+	log(logConfMembers, foundguys.size());
 
 	searchdest->redraw();
 	searchdest = 0;
@@ -805,8 +804,7 @@ void jabberhook::gotsearchresults(xmlnode x) {
 
     face.findready();
 
-    face.log(_("+ [jab] search finished, %d found"),
-	foundguys.size());
+    log(logSearchFinished, foundguys.size());
 
     searchdest->redraw();
     searchdest = 0;
@@ -844,9 +842,13 @@ void jabberhook::gotroster(xmlnode x) {
 	    ic = imcontact(jidtodisp(alias), proto);
 	    clist.updateEntry(ic, group ? group : "");
 
-	    if(name)
-	    if(c = clist.get(ic))
-		c->setdispnick(UTF2KOI(name));
+	    if(c = clist.get(ic)) {
+		if(name) c->setdispnick(rusconv("uk", name)); else {
+		    string u, h, r;
+		    jidsplit(alias, u, h, r);
+		    c->setdispnick(u);
+		}
+	    }
 
 	    roster[jidnormalize(alias)] = group ? group : "";
 	}
@@ -862,7 +864,7 @@ void jabberhook::postlogin() {
     flogged = true;
     ourstatus = available;
     setautostatus(jhook.manualstatus);
-    face.log(_("+ [jab] logged in"));
+    log(logLogged);
     face.update();
 
     for(i = 0; i < clist.count; i++) {
@@ -909,19 +911,19 @@ void jabberhook::conferencecreate(const imcontact &confid, const vector<imcontac
     xmlnode_free(x);
 }
 
-static void vcput(xmlnode x, const string &name, const string &val) {
+void jabberhook::vcput(xmlnode x, const string &name, const string &val) {
     xmlnode_insert_cdata(xmlnode_insert_tag(x, name.c_str()),
-	KOI2UTF(val).c_str(), (unsigned int) -1);
+	jhook.rusconv("ku", val).c_str(), (unsigned int) -1);
 }
 
-static void vcputphone(xmlnode x, const string &type, const string &place, const string &number) {
+void jabberhook::vcputphone(xmlnode x, const string &type, const string &place, const string &number) {
     xmlnode z = xmlnode_insert_tag(x, "TEL");
     vcput(z, type, "");
     vcput(z, place, "");
     vcput(z, "NUMBER", number);
 }
 
-static void vcputaddr(xmlnode x, const string &place, const string &street,
+void jabberhook::vcputaddr(xmlnode x, const string &place, const string &street,
 const string &locality, const string &region, const string &pcode,
 unsigned short country) {
     xmlnode z = xmlnode_insert_tag(x, "ADR");
@@ -930,7 +932,7 @@ unsigned short country) {
     vcput(z, "LOCALITY", locality);
     vcput(z, "REGION", region);
     vcput(z, "PCODE", pcode);
-    vcput(z, "CTRY", ICQ2000::UserInfoHelpers::getCountryIDtoString(country));
+    vcput(z, "CTRY", getCountryIDtoString(country));
 }
 
 void jabberhook::sendupdateuserinfo(const icqcontact &c) {
@@ -1028,7 +1030,7 @@ void jabberhook::gotmessage(const string &type, const string &from, const string
 	if(!r.empty()) body.insert(0, r + ": ");
     }
 
-    em.store(immessage(ic, imevent::incoming, UTF2KOI(body)));
+    em.store(immessage(ic, imevent::incoming, rusconv("uk", body)));
 }
 
 void jabberhook::updatecontact(icqcontact *c) {
@@ -1036,7 +1038,7 @@ void jabberhook::updatecontact(icqcontact *c) {
 
     if(logged()) {
 	auto_ptr<char> cjid(strdup(jidnormalize(c->getdesc().nickname).c_str()));
-	auto_ptr<char> cname(strdup(KOI2UTF(c->getdispnick()).c_str()));
+	auto_ptr<char> cname(strdup(rusconv("ku", c->getdispnick()).c_str()));
 
 	x = jutil_iqnew(JPACKET__SET, NS_ROSTER);
 	y = xmlnode_insert_tag(xmlnode_get_tag(x, "query"), "item");
@@ -1054,17 +1056,6 @@ void jabberhook::updatecontact(icqcontact *c) {
     }
 }
 
-static unsigned short getcountrybyname(string name) {
-    int i;
-    name = up(leadcut(trailcut(name)));    
-
-    for(i = 0; i < ICQ2000::Country_table_size; i++)
-	if(name == up(ICQ2000::Country_table[i].name))
-	    return ICQ2000::Country_table[i].code;
-
-    return 0;
-}
-
 void jabberhook::gotvcard(const imcontact &ic, xmlnode v) {
     xmlnode ad, n;
     char *p;
@@ -1081,7 +1072,7 @@ void jabberhook::gotvcard(const imcontact &ic, xmlnode v) {
 
 	for(ad = xmlnode_get_firstchild(v); ad; ad = xmlnode_get_nextsibling(ad)) {
 	    p = xmlnode_get_name(ad); name = p ? up(p) : "";
-	    p = xmlnode_get_data(ad); data = p ? UTF2KOI(p) : "";
+	    p = xmlnode_get_data(ad); data = p ? rusconv("uk", p) : "";
 
 	    if(name == "NICKNAME") c->setnick(data); else
 	    if(name == "DESC") c->setabout(data); else
@@ -1114,44 +1105,44 @@ void jabberhook::gotvcard(const imcontact &ic, xmlnode v) {
 		mi.birth_day = atoi(getword(data, "-").c_str());
 	    } else
 	    if(name == "ORG") {
-		if(p = xmlnode_get_tag_data(ad, "ORGNAME")) wi.company = UTF2KOI(p);
-		if(p = xmlnode_get_tag_data(ad, "ORGUNIT")) wi.dept = UTF2KOI(p);
+		if(p = xmlnode_get_tag_data(ad, "ORGNAME")) wi.company = rusconv("uk", p);
+		if(p = xmlnode_get_tag_data(ad, "ORGUNIT")) wi.dept = rusconv("uk", p);
 	    } else
 	    if(name == "N") {
-		if(p = xmlnode_get_tag_data(ad, "GIVEN")) bi.fname = UTF2KOI(p);
-		if(p = xmlnode_get_tag_data(ad, "FAMILY")) bi.lname = UTF2KOI(p);
+		if(p = xmlnode_get_tag_data(ad, "GIVEN")) bi.fname = rusconv("uk", p);
+		if(p = xmlnode_get_tag_data(ad, "FAMILY")) bi.lname = rusconv("uk", p);
 	    } else
 	    if(name == "ADR") {
 		if(xmlnode_get_tag(ad, "HOME")) {
-		    if(p = xmlnode_get_tag_data(ad, "STREET")) bi.street = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "LOCALITY")) bi.city = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "REGION")) bi.state = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "PCODE")) bi.zip = UTF2KOI(p);
+		    if(p = xmlnode_get_tag_data(ad, "STREET")) bi.street = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "LOCALITY")) bi.city = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "REGION")) bi.state = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "PCODE")) bi.zip = rusconv("uk", p);
 
 		    if((p = xmlnode_get_tag_data(ad, "CTRY"))
 		    || (p = xmlnode_get_tag_data(ad, "COUNTRY")))
-			bi.country = getcountrybyname(p);
+			bi.country = getCountryByName(p);
 
 		} else if(xmlnode_get_tag(ad, "WORK")) {
-		    if(p = xmlnode_get_tag_data(ad, "STREET")) wi.street = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "LOCALITY")) wi.city = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "REGION")) wi.state = UTF2KOI(p);
-		    if(p = xmlnode_get_tag_data(ad, "PCODE")) wi.zip = UTF2KOI(p);
+		    if(p = xmlnode_get_tag_data(ad, "STREET")) wi.street = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "LOCALITY")) wi.city = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "REGION")) wi.state = rusconv("uk", p);
+		    if(p = xmlnode_get_tag_data(ad, "PCODE")) wi.zip = rusconv("uk", p);
 
 		    if((p = xmlnode_get_tag_data(ad, "CTRY"))
 		    || (p = xmlnode_get_tag_data(ad, "COUNTRY")))
-			wi.country = getcountrybyname(p);
+			wi.country = getCountryByName(p);
 		}
 	    } else
 	    if(name == "TEL") {
 		if(p = xmlnode_get_tag_data(ad, "NUMBER")) {
 		    if(xmlnode_get_tag(ad, "VOICE")) {
-			if(xmlnode_get_tag(ad, "HOME")) bi.phone = UTF2KOI(p); else
-			if(xmlnode_get_tag(ad, "WORK")) wi.phone = UTF2KOI(p);
+			if(xmlnode_get_tag(ad, "HOME")) bi.phone = rusconv("uk", p); else
+			if(xmlnode_get_tag(ad, "WORK")) wi.phone = rusconv("uk", p);
 
 		    } else if(xmlnode_get_tag(ad, "FAX")) {
-			if(xmlnode_get_tag(ad, "HOME")) bi.fax = UTF2KOI(p); else
-			if(xmlnode_get_tag(ad, "WORK")) wi.fax = UTF2KOI(p);
+			if(xmlnode_get_tag(ad, "HOME")) bi.fax = rusconv("uk", p); else
+			if(xmlnode_get_tag(ad, "WORK")) wi.fax = rusconv("uk", p);
 		    }
 		}
 	    }
@@ -1174,22 +1165,6 @@ void jabberhook::gotvcard(const imcontact &ic, xmlnode v) {
     }
 }
 
-vector<icqcontact *> jabberhook::getneedsync() {
-    int i;
-    icqcontact *c;
-    vector<icqcontact *> r;
-
-    for(i = 0; i < clist.count; i++) {
-	c = (icqcontact *) clist.at(i);
-
-	if(c->getdesc().pname == proto)
-	if(roster.find(c->getdesc().nickname) == roster.end())
-	    r.push_back(c);
-    }
-
-    return r;
-}
-
 void jabberhook::requestversion(const imcontact &ic) {
     auto_ptr<char> cjid(strdup(jidnormalize(ic.nickname).c_str()));
     xmlnode x = jutil_iqnew(JPACKET__GET, NS_VERSION);
@@ -1207,18 +1182,18 @@ void jabberhook::gotversion(const imcontact &ic, xmlnode x) {
     if(y) {
 	if(z = xmlnode_get_tag(y, "name"))
 	if(p = xmlnode_get_data(y))
-	    if(p) vinfo = UTF2KOI(p);
+	    if(p) vinfo = rusconv("uk", p);
 
 	if(z = xmlnode_get_tag(y, "version"))
 	if(p = xmlnode_get_data(y)) {
 	    if(!vinfo.empty()) vinfo += ", ";
-	    vinfo += UTF2KOI(p);
+	    vinfo += rusconv("uk", p);
 	}
 
 	if(z = xmlnode_get_tag(y, "os"))
 	if(p = xmlnode_get_data(y)) {
 	    if(!vinfo.empty()) vinfo += " / ";
-	    vinfo += UTF2KOI(p);
+	    vinfo += rusconv("uk", p);
 	}
 
 	if(vinfo.size() > 128)
@@ -1267,7 +1242,7 @@ void jabberhook::statehandler(jconn conn, int state) {
 	case JCONN_STATE_OFF:
 	    if(previous_state != JCONN_STATE_OFF) {
 		logger.putourstatus(jhook.proto, jhook.getstatus(), jhook.ourstatus = offline);
-		face.log(_("+ [jab] disconnected"));
+		jhook.log(logDisconnected);
 		jhook.flogged = jhook.fonline = false;
 		jhook.roster.clear();
 		jhook.agents.clear();
@@ -1455,8 +1430,12 @@ void jabberhook::packethandler(jconn conn, jpacket packet) {
 		    case 504: /* Remote Server Timeout */
 		    default:
 			if(!jhook.regmode) {
-			    face.log(_("+ [jab] error %d: %s"), code, desc.c_str());
-			    if(!jhook.flogged) {
+			    face.log(desc.empty() ?
+				_("+ [jab] error %d") :
+				_("+ [jab] error %d: %s"),
+				code, desc.c_str());
+
+			    if(!jhook.flogged && code != 501) {
 				close(jhook.jc->fd);
 				jhook.jc->fd = -1;
 			    }
