@@ -1,7 +1,7 @@
 /*
 *
 * centericq livejournal protocol handling class (sick)
-* $Id: ljhook.cc,v 1.19 2003/11/22 19:14:34 konst Exp $
+* $Id: ljhook.cc,v 1.24 2004/02/01 17:52:10 konst Exp $
 *
 * Copyright (C) 2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -35,9 +35,6 @@
 
 ljhook lhook;
 
-#define KOI2UTF(x) siconv(x, conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset(), "utf-8")
-#define UTF2KOI(x) siconv(x, "utf-8", conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset())
-
 #define PERIOD_FRIENDS  3600
 
 ljhook::ljhook(): abstracthook(livejournal), fonline(false), sdest(0) {
@@ -63,10 +60,14 @@ void ljhook::connect() {
     baseurl = acc.server + ":" + i2str(acc.port) + "/interface/flat";
     md5pass = getmd5(acc.password);
 
-    face.log(_("+ [lj] connecting to the server"));
+    log(logConnecting);
 
     httpcli.setProxyServerHost(conf.gethttpproxyhost());
     httpcli.setProxyServerPort(conf.gethttpproxyport());
+    if (!conf.gethttpproxyuser().empty()) {
+	    httpcli.setProxyServerUser(conf.gethttpproxyuser());
+	    httpcli.setProxyServerPasswd(conf.gethttpproxypasswd());
+    }
 
     HTTPRequestEvent *ev = new HTTPRequestEvent(baseurl, HTTPRequestEvent::POST);
     ev->addParam("mode", "login");
@@ -98,7 +99,7 @@ void ljhook::connect() {
 void ljhook::disconnect() {
     if(fonline) {
 	fonline = false;
-	face.log(_("+ [lj] disconnected"));
+	log(logDisconnected);
 	if(flogged) {
 	    flogged = false;
 	    icqcontact *c = clist.get(self);
@@ -220,11 +221,20 @@ bool ljhook::enabled() const {
     return true;
 }
 
-bool ljhook::send(const imevent &sev) {
+bool ljhook::send(const imevent &asev) {
     if(!logged()) return false;
 
-    if(sev.gettype() == imevent::xml) {
-	const imxmlevent *m = static_cast<const imxmlevent *> (&sev);
+    bool ret = false;
+    imevent *sev;
+
+    if(asev.gettype() == imevent::xml) sev = asev.getevent(); else {
+	sev = new imxmlevent(asev.getcontact(), asev.getdirection(), asev.gettext());
+	static_cast<imxmlevent *>(sev)->setfield("_eventkind", "posting");
+
+    }
+
+    if(sev->gettype() == imevent::xml) {
+	const imxmlevent *m = static_cast<const imxmlevent *> (sev);
 	HTTPRequestEvent *ev = 0;
 
 	if(m->getfield("_eventkind") == "posting") {
@@ -233,7 +243,7 @@ bool ljhook::send(const imevent &sev) {
 	    ev->addParam("mode", "postevent");
 	    ev->addParam("user", username);
 	    ev->addParam("hpassword", md5pass);
-	    ev->addParam("event", KOI2UTF(m->gettext()));
+	    ev->addParam("event", rusconv("ku", m->gettext()));
 	    ev->addParam("ver", "1");
 
 	    time_t t = m->gettimestamp();
@@ -250,9 +260,9 @@ bool ljhook::send(const imevent &sev) {
 	    ev->addParam("allowmask", "0");
 
 	    if(!m->field_empty("journal")) ev->addParam("usejournal", m->getfield("journal"));
-	    if(!m->field_empty("subject")) ev->addParam("subject", KOI2UTF(m->getfield("subject")));
-	    if(!m->field_empty("mood")) ev->addParam("prop_current_mood", KOI2UTF(m->getfield("mood")));
-	    if(!m->field_empty("music")) ev->addParam("prop_current_music", KOI2UTF(m->getfield("music")));
+	    if(!m->field_empty("subject")) ev->addParam("subject", rusconv("ku", m->getfield("subject")));
+	    if(!m->field_empty("mood")) ev->addParam("prop_current_mood", rusconv("ku", m->getfield("mood")));
+	    if(!m->field_empty("music")) ev->addParam("prop_current_music", rusconv("ku", m->getfield("music")));
 	    if(!m->field_empty("picture")) ev->addParam("prop_picture_keyword", m->getfield("picture"));
 
 	    if(!m->field_empty("preformatted")) ev->addParam("prop_opt_preformatted", "1");
@@ -265,7 +275,7 @@ bool ljhook::send(const imevent &sev) {
 	} else if(m->getfield("_eventkind") == "comment") {
 	    icqconf::imaccount acc = conf.getourid(proto);
 
-	    string journal = clist.get(sev.getcontact())->getnick();
+	    string journal = clist.get(sev->getcontact())->getnick();
 	    journal.erase(journal.find("@"));
 
 	    ev = new HTTPRequestEvent(acc.server + ":" + i2str(acc.port) + "/talkpost_do.bml", HTTPRequestEvent::POST);
@@ -278,7 +288,7 @@ bool ljhook::send(const imevent &sev) {
 	    ev->addParam("password", acc.password);
 	    ev->addParam("do_login", "0");
 	    ev->addParam("subject", "");
-	    ev->addParam("body", KOI2UTF(sev.gettext()));
+	    ev->addParam("body", rusconv("ku", sev->gettext()));
 	    ev->addParam("subjecticon", "none");
 	    ev->addParam("prop_opt_preformatted", "1");
 	    ev->addParam("submitpost", "1");
@@ -289,11 +299,12 @@ bool ljhook::send(const imevent &sev) {
 
 	if(ev) {
 	    httpcli.SendEvent(ev);
-	    return true;
+	    ret = true;
 	}
     }
 
-    return false;
+    delete sev;
+    return ret;
 }
 
 void ljhook::sendnewuser(const imcontact &ic) {
@@ -514,7 +525,7 @@ void ljhook::messageack_cb(MessageEvent *ev) {
 
 	if(params["success"] == "OK") {
 	    flogged = true;
-	    face.log(_("+ [lj] logged in"));
+	    log(logLogged);
 	    face.update();
 	    setautostatus(manualstatus);
 
@@ -562,14 +573,19 @@ void ljhook::messageack_cb(MessageEvent *ev) {
 	if(params["success"] == "OK") {
 	    journals = vector<string>(1, username);
 
-	    count = atoi(params["friend_count"].c_str());
+	    if(conf.getourid(proto).additional["importfriends"] == "1") {
+		count = atoi(params["friend_count"].c_str());
+	    } else {
+		count = 0;
+	    }
+
 	    string username, name, birthday, bd;
 	    icqcontact::moreinfo mi;
 	    icqcontact::basicinfo bi;
 
 	    for(i = 1; i <= count; i++) {
 		username = params[(string) "friend_" + i2str(i) + "_user"];
-		name = UTF2KOI(params[(string) "friend_" + i2str(i) + "_name"]);
+		name = rusconv("uk", params[(string) "friend_" + i2str(i) + "_name"]);
 		birthday = params[(string) "friend_" + i2str(i) + "_birthday"];
 
 		bool found = false;
@@ -629,7 +645,7 @@ void ljhook::messageack_cb(MessageEvent *ev) {
 
 	    for(i = 1; i <= count; i++) {
 		username = params[(string) "friendof_" + i2str(i) + "_user"];
-		name = UTF2KOI(params[(string) "friendof_" + i2str(i) + "_name"]);
+		name = rusconv("uk", params[(string) "friendof_" + i2str(i) + "_name"]);
 		nfriendof[username] = name;
 	    }
 
