@@ -1,9 +1,9 @@
 /*
 *
 * centericq yahoo! protocol handling class
-* $Id: yahoohook.cc,v 1.98 2003/11/23 01:09:35 konst Exp $
+* $Id: yahoohook.cc,v 1.105 2004/02/22 13:03:59 konst Exp $
 *
-* Copyright (C) 2003 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2003-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,9 @@
 #include <netinet/in.h>
 
 #define PERIOD_REFRESH          60
+#define PERIOD_CLOSE            6
+
+int yahoohook::yfd::connection_tags = 0;
 
 char pager_host[255], pager_port[255], filetransfer_host[255],
     filetransfer_port[255], webcam_host[255], webcam_port[255],
@@ -67,7 +70,6 @@ static int stat2int[imstatus_size] = {
 yahoohook::yahoohook() : abstracthook(yahoo), fonline(false), cid(0) {
     fcapabs.insert(hookcapab::setaway);
     fcapabs.insert(hookcapab::fetchaway);
-    fcapabs.insert(hookcapab::synclist);
     fcapabs.insert(hookcapab::directadd);
     fcapabs.insert(hookcapab::files);
     fcapabs.insert(hookcapab::conferencing);
@@ -122,8 +124,8 @@ void yahoohook::init() {
     c.ext_yahoo_webcam_closed = &webcam_closed;
     c.ext_yahoo_webcam_viewer = &webcam_viewer;
     c.ext_yahoo_webcam_data_request = &webcam_data_request;
-    c.ext_yahoo_search_result = &search_result;
-    c.ext_yahoo_log = &log;
+    c.ext_yahoo_got_search_result = &got_search_result;
+    c.ext_yahoo_log = &ylog;
 
     yahoo_register_callbacks(&c);
 }
@@ -144,7 +146,7 @@ void yahoohook::connect() {
     strcpy(local_host, "");
     conn_type = 1;
 
-    face.log(_("+ [yahoo] connecting to the server"));
+    log(logConnecting);
 
     if(cid < 0) {
 	yahoo_logoff(cid);
@@ -247,6 +249,7 @@ bool yahoohook::isoursocket(fd_set &rs, fd_set &ws, fd_set &es) const {
 void yahoohook::disconnect() {
     if(online()) {
 	yahoo_logoff(cid);
+	time(&timer_close);
     }
 }
 
@@ -255,7 +258,7 @@ void yahoohook::disconnected() {
 	logger.putourstatus(proto, getstatus(), ourstatus = offline);
 	clist.setoffline(proto);
 	fonline = false;
-	face.log(_("+ [yahoo] disconnected"));
+	log(logDisconnected);
 	face.update();
     }
 }
@@ -278,6 +281,11 @@ void yahoohook::exectimers() {
 	if(timer_current-timer_refresh > PERIOD_REFRESH) {
 	    yahoo_refresh(cid);
 	    timer_refresh = timer_current;
+
+	} else if(timer_close && timer_current-timer_close > PERIOD_CLOSE) {
+	    yahoo_close(cid);
+	    disconnected();
+
 	}
     }
 }
@@ -289,7 +297,6 @@ struct tm *yahoohook::timestamp() {
 bool yahoohook::send(const imevent &ev) {
     icqcontact *c = clist.get(ev.getcontact());
     string text;
-    string::iterator is;
 
     if(c) {
 	if(ev.gettype() == imevent::message) {
@@ -357,9 +364,7 @@ void yahoohook::sendnewuser(const imcontact &ic, bool report) {
 		found = ic.nickname == static_cast<yahoo_buddy *>(bud->data)->id;
 
 	    if(!found) {
-		if(report)
-		    face.log(_("+ [yahoo] adding %s to the contacts"), ic.nickname.c_str());
-
+		if(report) log(logContactAdd, ic.nickname.c_str());
 		vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), clist.get(ic)->getgroupid());
 		if(ig != groups.end()) {
 		    yahoo_add_buddy(cid, ic.nickname.c_str(), ig->getname().c_str());
@@ -378,9 +383,7 @@ void yahoohook::removeuser(const imcontact &ic) {
 void yahoohook::removeuser(const imcontact &ic, bool report) {
     if(logged()) {
 	if(!ischannel(ic)) {
-	    if(report)
-		face.log(_("+ [yahoo] removing %s from the contacts"),
-		    ic.nickname.c_str());
+	    if(report) log(logContactRemove, ic.nickname.c_str());
 
 	    const YList *buddies = yahoo_get_buddylist(cid);
 	    const YList *bud = 0;
@@ -527,37 +530,10 @@ YList *yahoohook::getmembers(const string &room) {
     return smemb;
 }
 
-vector<icqcontact *> yahoohook::getneedsync() {
-    int i;
-    icqcontact *c;
-    vector<icqcontact *> r;
-    bool found;
-
-    const YList *buddies = yahoo_get_buddylist(cid);
-    const YList *bud = 0;
-
-    for(i = 0; i < clist.count; i++) {
-	c = (icqcontact *) clist.at(i);
-
-	if(c->getdesc().pname == proto) {
-	    for(found = false, bud = buddies; bud && !found; bud = y_list_next(bud))
-		found = c->getdesc().nickname == static_cast<yahoo_buddy *>(bud->data)->id;
-
-	    if(!found)
-		r.push_back(c);
-	}
-    }
-
-    return r;
-}
-
 string yahoohook::decode(string text, bool utf) {
     int npos, mpos;
 
-    if(utf) text = siconv(text, "utf-8",
-	conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset());
-
-    text = rushtmlconv("wk", text);
+    text = rushtmlconv(utf ? "uk" : "wk", text);
 
     while((npos = text.find("\e[")) != -1) {
 	if((mpos = text.substr(npos).find("m")) == -1)
@@ -600,16 +576,23 @@ void yahoohook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	foundguys.pop_back();
     }
 
+    yahoo_search_gender gender = YAHOO_GENDER_NONE;
+
+    switch(params.gender) {
+	case genderMale: gender = YAHOO_GENDER_MALE; break;
+	case genderFemale: gender = YAHOO_GENDER_FEMALE; break;
+    }
+
     searchonlineonly = params.onlineonly;
     searchdest = &dest;
 
     if(!params.kwords.empty()) {
 	yahoo_search(cid, YAHOO_SEARCH_KEYWORD, decode(params.kwords, false).c_str(),
-		params.gender, YAHOO_AGERANGE_NONE, params.photo ? 1 : 0, 1);
+		gender, YAHOO_AGERANGE_NONE, params.photo ? 1 : 0, 1);
 
     } else if(!params.firstname.empty()) {
 	yahoo_search(cid, YAHOO_SEARCH_NAME, decode(params.firstname, false).c_str(),
-		params.gender, YAHOO_AGERANGE_NONE, params.photo ? 1 : 0, 1);
+		gender, YAHOO_AGERANGE_NONE, params.photo ? 1 : 0, 1);
 
     } else if(!params.room.empty()) {
 	room = params.room.substr(1);
@@ -624,8 +607,7 @@ void yahoohook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	}
 
 	face.findready();
-	face.log(_("+ [yahoo] members list fetching finished, %d found"),
-	    searchdest->getcount());
+	log(logConfMembers, searchdest->getcount());
 
 	searchdest->redraw();
 	searchdest = 0;
@@ -670,13 +652,21 @@ void yahoohook::requestawaymsg(const imcontact &ic) {
 }
 
 void yahoohook::checkinlist(imcontact ic) {
+    bool found = false;
     icqcontact *c = clist.get(ic);
-    vector<icqcontact *> notremote = getneedsync();
 
     if(c)
-    if(c->inlist())
-    if(find(notremote.begin(), notremote.end(), c) != notremote.end())
-	sendnewuser(ic, false);
+    if(c->inlist()) {
+	const YList *buddies = yahoo_get_buddylist(cid);
+
+	for(const YList *bud = buddies; bud && !found; ) {
+	    yahoo_buddy *yb = (yahoo_buddy *) bud->data;
+	    found = (c->getdesc().nickname == yb->id);
+	    bud = y_list_next(bud);
+	}
+
+	if(!found) sendnewuser(ic, false);
+    }
 }
 
 void yahoohook::updatecontact(icqcontact *c) {
@@ -721,9 +711,10 @@ void yahoohook::login_response(int id, int succ, char *url) {
 	case YAHOO_LOGIN_OK:
 	    yhook.flogged = true;
 	    logger.putourstatus(yahoo, offline, yhook.ourstatus = yhook.manualstatus);
-	    face.log(_("+ [yahoo] logged in"));
+	    yhook.log(logLogged);
 	    time(&yhook.timer_refresh);
 	    yhook.setautostatus(yhook.manualstatus);
+	    yhook.timer_close = 0;
 	    break;
 
 	case YAHOO_LOGIN_PASSWD:
@@ -745,6 +736,7 @@ void yahoohook::login_response(int id, int succ, char *url) {
 	    break;
 
 	case YAHOO_LOGIN_SOCK:
+	    face.log(_("+ [yahoo] server closed socket"));
 	    yhook.disconnected();
 	    break;
     }
@@ -770,17 +762,17 @@ void yahoohook::status_changed(int id, char *who, int stat, char *msg, int away)
 
 void yahoohook::got_im(int id, char *who, char *msg, long tm, int stat, int utf8) {
     imcontact ic(who, yahoo);
-    string text = cuthtml(msg, true);
+    string text = cuthtml(msg, chCutBR | chLeaveLinks);
 
     yhook.checkinlist(ic);
     text = yhook.decode(text, utf8);
 
     if(!text.empty()) {
-	em.store(immessage(ic, imevent::incoming, text));
+	em.store(immessage(ic, imevent::incoming, text, tm));
     }
 }
 
-void yahoohook::search_result(int id, struct yahoo_search_result *yr) {
+void yahoohook::got_search_result(int id, int found, int start, int total, YList *contacts) {
     yahoo_found_contact *fc;
     icqcontact *c;
     YList *ir;
@@ -789,7 +781,7 @@ void yahoohook::search_result(int id, struct yahoo_search_result *yr) {
     if(!yhook.searchdest)
 	return;
 
-    for(ir = yr->contacts; ir; ir = ir->next) {
+    for(ir = contacts; ir; ir = ir->next) {
 	fc = (yahoo_found_contact *) ir->data;
 
 	if(yhook.searchonlineonly && !fc->online)
@@ -841,12 +833,12 @@ void yahoohook::search_result(int id, struct yahoo_search_result *yr) {
 
     yhook.searchdest->redraw();
 
-    if(yr->start+yr->found >= yr->total) {
+    if(start + found >= total) {
 	face.findready();
-	face.log(_("+ [yahoo] search finished, %d found"), yhook.foundguys.size());
+	yhook.log(logSearchFinished, yhook.foundguys.size());
 	yhook.searchdest = 0;
     } else {
-	yahoo_search_again(yhook.cid);
+	yahoo_search_again(yhook.cid, -1);
     }
 }
 
@@ -937,7 +929,7 @@ void yahoohook::conf_userleave(int id, char *who, char *room) {
 void yahoohook::conf_message(int id, char *who, char *room, char *msg, int utf8) {
     icqcontact *c = clist.get(imcontact((string) "#" + room, yahoo));
 
-    string text = (string) who + ": " + cuthtml(msg, true);
+    string text = (string) who + ": " + cuthtml(msg, chCutBR | chLeaveLinks);
     text = yhook.decode(text, utf8);
 
     if(c) em.store(immessage(c, imevent::incoming, text));
@@ -1000,21 +992,31 @@ void yahoohook::error(int id, char *err, int fatal) {
     }
 }
 
-void yahoohook::add_handler(int id, int fd, yahoo_input_condition cond, void *data) {
+int yahoohook::add_handler(int id, int fd, yahoo_input_condition cond, void *data) {
+    int tag = -1;
+
     switch(cond) {
-	case YAHOO_INPUT_READ: yhook.rfds.push_back(yfd(fd, data)); break;
-	case YAHOO_INPUT_WRITE: yhook.wfds.push_back(yfd(fd, data)); break;
+	case YAHOO_INPUT_READ:
+	    yhook.rfds.push_back(yfd(fd, data));
+	    tag = yhook.rfds.back().tag;
+	    break;
+	case YAHOO_INPUT_WRITE:
+	    yhook.wfds.push_back(yfd(fd, data));
+	    tag = yhook.wfds.back().tag;
+	    break;
     }
+
+    return tag;
 }
 
-void yahoohook::remove_handler(int id, int fd) {
+void yahoohook::remove_handler(int tag) {
     vector<yfd>::iterator i;
 
-    i = find(yhook.rfds.begin(), yhook.rfds.end(), fd);
+    i = find(yhook.rfds.begin(), yhook.rfds.end(), tag);
     if(i != yhook.rfds.end())
 	yhook.rfds.erase(i);
 
-    i = find(yhook.wfds.begin(), yhook.wfds.end(), fd);
+    i = find(yhook.wfds.begin(), yhook.wfds.end(), tag);
     if(i != yhook.wfds.end())
 	yhook.wfds.erase(i);
 }
@@ -1055,6 +1057,7 @@ int yahoohook::connect_async(int id, char *host, int port, yahoo_connect_callbac
 	ccd->id = id;
 
 	yhook.wfds.push_back(yfd(servfd, ccd, true));
+	ccd->tag = yhook.wfds.back().tag;
 	return 1;
     } else {
 	close(servfd);
@@ -1064,11 +1067,12 @@ int yahoohook::connect_async(int id, char *host, int port, yahoo_connect_callbac
 
 void yahoohook::connect_complete(void *data, int source) {
     connect_callback_data *ccd = (connect_callback_data *) data;
-    int error, err_size = sizeof(error);
+    char error;
+    socklen_t err_size = sizeof(error);
 
-    remove_handler(-1, source);
+    remove_handler(ccd->tag);
 
-    if(getsockopt(source, SOL_SOCKET, SO_ERROR, &error, (socklen_t *) &err_size) == -1 || error != 0) {
+    if(getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &err_size) == -1 || error != 0) {
 	if(yhook.logged())
 	    face.log(_("+ [yahoo] direct connection failed"));
 
@@ -1090,7 +1094,7 @@ void yahoohook::got_cookies(int id) {
 void yahoohook::chat_cat_xml(int id, char *xml) {
 }
 
-void yahoohook::chat_join(int id, char *room, char *topic, YList *members) {
+void yahoohook::chat_join(int id, char *room, char *topic, YList *members, int fd) {
 }
 
 void yahoohook::chat_userjoin(int id, char *room, struct yahoo_chat_member *who) {
@@ -1123,7 +1127,7 @@ void yahoohook::webcam_viewer(int id, char *who, int connect) {
 void yahoohook::webcam_data_request(int id, int send) {
 }
 
-int yahoohook::log(char *fmt, ...) {
+int yahoohook::ylog(char *fmt, ...) {
     return 0;
 }
 
