@@ -10,7 +10,6 @@
 #include <map>
 #include <set>
 #include <memory>
-
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -19,6 +18,12 @@
 
 #include <config.h>
 
+#ifdef HAVE_SSTREAM
+    #include <sstream>
+#else
+    #include <strstream>
+#endif
+
 #include "kkiproc.h"
 #include "kkstrtext.h"
 #include "conf.h"
