@@ -1,7 +1,7 @@
 /*
 *
 * centericq single IM contact class
-* $Id: icqcontact.cc,v 1.88 2003/11/05 14:54:26 konst Exp $
+* $Id: icqcontact.cc,v 1.94 2004/03/15 22:36:36 konst Exp $
 *
 * Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -31,8 +31,11 @@
 #include "eventmanager.h"
 
 #include <time.h>
-#include <libicq2000/userinfohelpers.h>
+#ifdef HAVE_SSTREAM
+#include <sstream>
+#else
 #include <strstream>
+#endif
 
 icqcontact::icqcontact(const imcontact adesc) {
     string fname, tname;
@@ -94,6 +97,7 @@ string icqcontact::getdirname() const {
 	case icq:
 	case infocard:
 	case rss:
+	case gadu:
 	    ret += conf.getprotocolprefix(cdesc.pname) + i2str(cdesc.uin);
 	    break;
 	default:
@@ -389,6 +393,13 @@ void icqcontact::excludefromlist() {
     finlist = false;
 }
 
+void icqcontact::includeintolist(int agroupid, bool reqauth) {
+    binfo.requiresauth = binfo.authawait = reqauth;
+    if(groupid) groupid = agroupid;
+
+    includeintolist();
+}
+
 void icqcontact::includeintolist() {
     if(cdesc.pname != icq)
 	status = offline;
@@ -410,7 +421,11 @@ void icqcontact::scanhistory() {
     int pos, backstep, r;
     FILE *f = fopen(fn.c_str(), "r");
     struct stat st;
+#ifdef HAVE_SSTREAM
+    stringstream evdata;
+#else
     strstream evdata;
+#endif
 
     sethasevents(false);
 
@@ -438,8 +453,8 @@ void icqcontact::scanhistory() {
 		block.erase(0, r+2);
 		evdata << block;
 
-		for(r = 0; (r < 4) && getline(evdata, block); r++);
-		if(r == 4) sethasevents(strtoul(block.c_str(), 0, 0) > lastread);
+		for(r = 0; (r < 3) && getline(evdata, block); r++);
+		if(r == 3) sethasevents(strtoul(block.c_str(), 0, 0) > lastread);
 		break;
 	    }
 	}
@@ -624,6 +639,7 @@ time_t icqcontact::getlastread() const {
 }
 
 imstatus icqcontact::getstatus() const {
+    if(conf.getnonimonline(cdesc.pname)) return available;
     return status;
 }
 
@@ -651,7 +667,7 @@ time_t icqcontact::getlastseen() const {
 
 char icqcontact::getshortstatus() const {
     if(status >= offline && status < imstatus_size) {
-	return imstatus2char[status];
+	return imstatus2char[getstatus()];
     } else {
 	return imstatus2char[offline];
     }
@@ -726,8 +742,8 @@ string icqcontact::moreinfo::strtimezone() const {
     string r;
 
     if(timezone <= 24 && timezone >= -24) {
-	r = ICQ2000::UserInfoHelpers::getTimezoneIDtoString(timezone) + ", " +
-	    ICQ2000::UserInfoHelpers::getTimezonetoLocaltime(timezone);
+	r = abstracthook::getTimezoneIDtoString(timezone) + ", " +
+	    abstracthook::getTimezonetoLocaltime(timezone);
     }
 
     return r;
