@@ -1,9 +1,9 @@
 /*
 *
 * centericq user interface class, dialogs related part
-* $Id: icqdialogs.cc,v 1.133 2003/11/22 19:14:32 konst Exp $
+* $Id: icqdialogs.cc,v 1.141 2004/03/09 21:46:26 konst Exp $
 *
-* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -32,17 +32,16 @@
 #include "abstracthook.h"
 #include "ljhook.h"
 
-#include <libicq2000/userinfohelpers.h>
-#include <libicq2000/constants.h>
+#include <libicq2000/userinfoconstants.h>
 
 const char *stragerange(ICQ2000::AgeRange r) {
     switch(r) {
-	case ICQ2000::range_18_22: return "18-22";
-	case ICQ2000::range_23_29: return "23-29";
-	case ICQ2000::range_30_39: return "30-39";
-	case ICQ2000::range_40_49: return "40-49";
-	case ICQ2000::range_50_59: return "50-59";
-	case ICQ2000::range_60_above: return _("60-above");
+	case ICQ2000::RANGE_18_22: return "18-22";
+	case ICQ2000::RANGE_23_29: return "23-29";
+	case ICQ2000::RANGE_30_39: return "30-39";
+	case ICQ2000::RANGE_40_49: return "40-49";
+	case ICQ2000::RANGE_50_59: return "50-59";
+	case ICQ2000::RANGE_60_ABOVE: return _("60-above");
     }
 
     return "";
@@ -50,16 +49,16 @@ const char *stragerange(ICQ2000::AgeRange r) {
 
 const char *strrandomgroup(short unsigned int gr) {
     switch((ICQ2000::RandomChatGroup) gr) {
-	case ICQ2000::group_GeneralChat: return _("General Chat");
-	case ICQ2000::group_Romance: return _("Romance");
-	case ICQ2000::group_Games: return _("Games");
-	case ICQ2000::group_Students: return _("Students");
-	case ICQ2000::group_20: return _("20 Something");
-	case ICQ2000::group_30: return _("30 Something");
-	case ICQ2000::group_40: return _("40 Something");
-	case ICQ2000::group_50Plus: return _("50 Plus");
-	case ICQ2000::group_SeekingWomen: return _("Seeking Women");
-	case ICQ2000::group_SeekingMen: return _("Seeking Men");
+	case ICQ2000::GROUP_GENERALCHAT: return _("General Chat");
+	case ICQ2000::GROUP_ROMANCE: return _("Romance");
+	case ICQ2000::GROUP_GAMES: return _("Games");
+	case ICQ2000::GROUP_STUDENTS: return _("Students");
+	case ICQ2000::GROUP_20: return _("20 Something");
+	case ICQ2000::GROUP_30: return _("30 Something");
+	case ICQ2000::GROUP_40: return _("40 Something");
+	case ICQ2000::GROUP_50PLUS: return _("50 Plus");
+	case ICQ2000::GROUP_SEEKINGWOMEN: return _("Seeking Women");
+	case ICQ2000::GROUP_SEEKINGMEN: return _("Seeking Men");
     }
 
     return "";
@@ -282,44 +281,60 @@ bool icqface::finddialog(imsearchparams &s, findsubject subj) {
 	if(subj != fschannel)
 	switch(s.pname) {
 	    case icq:
+	    case gadu:
 		i = tree.addnode(_(" UIN "));
 		tree.addleaff(i, 0, 10, " %s ", strint(s.uin));
 
 		if(!s.uin && !s.randomgroup && s.kwords.empty()) {
 		    i = tree.addnode(_(" Details "));
 		    tree.addleaff(i, 0, 11, _(" Nickname : %s "), s.nick.c_str());
-		    tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
+
+		    if(s.pname == icq) {
+			tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
+		    }
+
 		    tree.addleaff(i, 0, 13, _(" First name : %s "), s.firstname.c_str());
 		    tree.addleaff(i, 0, 14, _(" Last name : %s "), s.lastname.c_str());
 
-		    tree.addleaff(i, 0, 15, _(" Age range : %s "), stragerange(s.agerange));
+		    if(s.pname == icq) {
+			tree.addleaff(i, 0, 15, _(" Age range : %s "), stragerange(s.agerange));
+		    }
+
 		    tree.addleaff(i, 0, 17, _(" Gender : %s "), strgender(s.gender));
-		    tree.addleaff(i, 0, 18, _(" Language : %s "),
-			ICQ2000::UserInfoHelpers::getLanguageIDtoString(s.language).c_str());
+
+		    if(s.pname == icq) {
+			tree.addleaff(i, 0, 18, _(" Language : %s "),
+			    abstracthook::getLanguageIDtoString(s.language).c_str());
+		    }
 
 		    i = tree.addnode(_(" Location "));
 		    tree.addleaff(i, 0, 19, _(" City : %s "), s.city.c_str());
-		    tree.addleaff(i, 0, 20, _(" State : %s "), s.state.c_str());
-		    tree.addleaff(i, 0, 21, _(" Country : %s "),
-			ICQ2000::UserInfoHelpers::getCountryIDtoString(s.country).c_str());
 
-		    i = tree.addnode(_(" Work "));
-		    tree.addleaff(i, 0, 22, _(" Company : %s "), s.company.c_str());
-		    tree.addleaff(i, 0, 23, _(" Department : %s "), s.department.c_str());
-		    tree.addleaff(i, 0, 24, _(" Position : %s "), s.position.c_str());
+		    if(s.pname == icq) {
+			tree.addleaff(i, 0, 20, _(" State : %s "), s.state.c_str());
+			tree.addleaff(i, 0, 21, _(" Country : %s "),
+			    abstracthook::getCountryIDtoString(s.country).c_str());
+
+			i = tree.addnode(_(" Work "));
+			tree.addleaff(i, 0, 22, _(" Company : %s "), s.company.c_str());
+			tree.addleaff(i, 0, 23, _(" Department : %s "), s.department.c_str());
+			tree.addleaff(i, 0, 24, _(" Position : %s "), s.position.c_str());
+		    }
 
 		    i = tree.addnode(_(" Online only "));
 		    tree.addleaff(i, 0, 25, " %s ", stryesno(s.onlineonly));
 		}
 
-		if(!s.uin && s.kwords.empty()) {
-		    i = tree.addnode(_(" Random chat group "));
-		    tree.addleaff(i, 0, 28, " %s ", strrandomgroup(s.randomgroup));
-		} 
+		if(s.pname == icq) {
+		    if(!s.uin && s.kwords.empty()) {
+			i = tree.addnode(_(" Random chat group "));
+			tree.addleaff(i, 0, 28, " %s ", strrandomgroup(s.randomgroup));
+		    }
 
-		if(!s.uin && !s.randomgroup) {
-		    i = tree.addnode(_(" Keywords "));
-		    tree.addleaff(i, 0, 29, " %s ", s.kwords.c_str());
+		    if(!s.uin && !s.randomgroup) {
+			i = tree.addnode(_(" Keywords "));
+			tree.addleaff(i, 0, 29, " %s ", s.kwords.c_str());
+		    }
 		}
 		break;
 
@@ -504,7 +519,11 @@ bool icqface::finddialog(imsearchparams &s, findsubject subj) {
 		break;
 
 	    case 4:
-		ret = finished = true;
+		if(s.pname == jabber && s.nick.find("@") == -1) {
+		    status(_("Wrong Jabber ID!"));
+		} else {
+		    ret = finished = true;
+		}
 		break;
 	}
     }
@@ -598,7 +617,7 @@ void icqface::gendetails(treeview *tree, icqcontact *c) {
 	tree->addleaff(i, 0, 17, _(" City : %s "), bi.city.c_str());
 	tree->addleaff(i, 0, 18, _(" State : %s "), bi.state.c_str());
 	tree->addleaff(i, 0, 19, _(" Country : %s "),
-	    ICQ2000::UserInfoHelpers::getCountryIDtoString(bi.country).c_str());
+	    abstracthook::getCountryIDtoString(bi.country).c_str());
 
 	tree->addleaff(i, 0, 20, _(" Street address : %s "), bi.street.c_str());
 	tree->addleaff(i, 0, 21, _(" Zip code : %s "), bi.zip.c_str());
@@ -610,7 +629,7 @@ void icqface::gendetails(treeview *tree, icqcontact *c) {
 	tree->addleaff(i, 0, 25, _(" City : %s "), wi.city.c_str());
 	tree->addleaff(i, 0, 26, _(" State : %s "), wi.state.c_str());
 	tree->addleaff(i, 0, 27, _(" Country : %s "),
-	    ICQ2000::UserInfoHelpers::getCountryIDtoString(wi.country).c_str());
+	    abstracthook::getCountryIDtoString(wi.country).c_str());
 
 	tree->addleaff(i, 0, 28, _(" Street address : %s "), wi.street.c_str());
 	tree->addleaff(i, 0, 41, _(" Zip code : %s "), wi.zip.c_str());
@@ -626,19 +645,18 @@ void icqface::gendetails(treeview *tree, icqcontact *c) {
 
 	if(passinfo.pname == icq) {
 	    tree->addleaff(i, 0, 36, _(" 1st language : %s "),
-		ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang1).c_str());
+		abstracthook::getLanguageIDtoString(mi.lang1).c_str());
 
 	    tree->addleaff(i, 0, 37, _(" 2nd language : %s "),
-		ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang2).c_str());
+		abstracthook::getLanguageIDtoString(mi.lang2).c_str());
 
 	    tree->addleaff(i, 0, 38, _(" 3rd language : %s "),
-		ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang3).c_str());
+		abstracthook::getLanguageIDtoString(mi.lang3).c_str());
 
 	    i = tree->addnode(_(" Miscellaneous "));
 	    if(ourdetails) {
 		tree->addleaff(i, 0, 43, _(" Enable web status indicator : %s "), stryesno(bi.webaware));
 		tree->addleaff(i, 0, 42, _(" Random chat group : %s "), strrandomgroup(bi.randomgroup));
-		tree->addleaff(i, 0, 45, _(" Store contact list server-side : %s "), stryesno(bi.autosync));
 	    } else {
 		tree->addleaff(i, 0, 44, _(" Authorization required : %s "), stryesno(bi.requiresauth));
 	    }
@@ -798,7 +816,6 @@ bool icqface::updatedetails(icqcontact *c, protocolname upname) {
 		case 42: selectrandomgroup(bi.randomgroup); break;
 		case 43: bi.webaware = !bi.webaware; break;
 		case 44: bi.requiresauth = !bi.requiresauth; break;
-		case 45: bi.autosync = !bi.autosync; break;
 		case 46: {
 		    abstracthook &h = gethook(passinfo.pname);
 		    vector<string> servs = h.getservices(servicetype::registration);
@@ -857,11 +874,11 @@ void icqface::selectcountry(unsigned short &f) {
     m.setwindow(textwindow(4, LINES-16, 30, LINES-3, conf.getcolor(cp_dialog_menu)));
     m.idle = &menuidle;
 
-    for(i = 0; i < ICQ2000::Country_table_size; i++) {
-	m.additem(0, (int) ICQ2000::Country_table[i].code,
-	    (string) " " + ICQ2000::Country_table[i].name);
+    for(i = 0; i < abstracthook::Country_table_size; i++) {
+	m.additem(0, (int) abstracthook::Country_table[i].code,
+	    (string) " " + abstracthook::Country_table[i].name);
 
-	if(ICQ2000::Country_table[i].code == f) {
+	if(abstracthook::Country_table[i].code == f) {
 	    m.setpos(i);
 	}
     }
@@ -878,8 +895,8 @@ void icqface::selectlanguage(unsigned short &f) {
     m.setwindow(textwindow(4, LINES-13, 20, LINES-4, conf.getcolor(cp_dialog_menu)));
     m.idle = &menuidle;
 
-    for(i = 0; i < ICQ2000::Language_table_size; i++) {
-	m.additemf(" %s", ICQ2000::Language_table[i]);
+    for(i = 0; i < abstracthook::Language_table_size; i++) {
+	m.additemf(" %s", abstracthook::Language_table[i]);
     }
 
     m.setpos(f);
@@ -897,7 +914,7 @@ void icqface::selectrandomgroup(unsigned short &f) {
 
     m.additemf(" %s", _("none"));
 
-    for(i = 1; i <= ((int) ICQ2000::group_SeekingMen); i++) {
+    for(i = 1; i <= ((int) ICQ2000::GROUP_SEEKINGMEN); i++) {
 	m.additemf(" %s", strrandomgroup(i));
     }
 
@@ -929,10 +946,10 @@ void icqface::selectagerange(ICQ2000::AgeRange &r) {
     m.setwindow(textwindow(4, LINES-8, 18, LINES-3, conf.getcolor(cp_dialog_menu)));
     m.idle = &menuidle;
 
-    for(ICQ2000::AgeRange i = ICQ2000::range_NoRange; i <= ICQ2000::range_60_above; (int) i += 1) {
+    for(ICQ2000::AgeRange i = ICQ2000::RANGE_NORANGE; i <= ICQ2000::RANGE_60_ABOVE; (int) i += 1) {
 	const char *p = stragerange(i);
 
-	if(i == ICQ2000::range_NoRange)
+	if(i == ICQ2000::RANGE_NORANGE)
 	    p = _("none");
 
 	m.additemf(0, (int) i, " %s", p);
@@ -1046,6 +1063,19 @@ bool icqface::sendfiles(const imcontact &cinfo, string &msg, linkedlist &flist)
     return flist.count;
 }
 
+void icqface::multichange(bool conv[], bool newstate) {
+    protocolname pname;
+
+    for(pname = icq; pname != protocolname_size; (int) pname += 1)
+	if((!conf.getourid(pname).empty() || pname == rss) && conv[pname]) break;
+
+    if(pname == protocolname_size || !newstate) {
+	for(pname = icq; pname != protocolname_size; (int) pname += 1)
+	    if(!conf.getourid(pname).empty() || pname == rss)
+		conv[pname] = newstate;
+    }
+}
+
 bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     bool finished, success, hasany;
     int nopt, n, i, b, nconf, ncomm, aaway, ana, noth, nfeat, ncl;
@@ -1071,20 +1101,36 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     conf.getpeertopeer(ptpmin, ptpmax);
     bool ptp = ptpmax;
 
-    string httpproxy = conf.gethttpproxyhost();
-    if(!httpproxy.empty()) httpproxy += ":" + i2str(conf.gethttpproxyport());
+    string httpproxy;
+    if (!conf.gethttpproxyuser().empty()) {
+	httpproxy = conf.gethttpproxyuser();
+	if(!httpproxy.empty())
+	    httpproxy += ":" + conf.gethttpproxypasswd() + "@" + conf.gethttpproxyhost() + ":" + i2str(conf.gethttpproxyport());
+    } else {
+	httpproxy = conf.gethttpproxyhost();
+	if(!httpproxy.empty())
+	    httpproxy += ":" + i2str(conf.gethttpproxyport());
+    }
 
-    string defcharset = conf.getdefcharset();
+    vector<string> convlanguages;
+    convlanguages.push_back(_("None"));
+    convlanguages.push_back(_("Russian"));
+    convlanguages.push_back(_("Polish"));
+    vector<string>::const_iterator iconvlang = convlanguages.begin();
+
+    string convertfrom = conf.getconvertfrom();
+    string convertto = conf.getconvertto();
 
     icqconf::groupmode gmode = conf.getgroupmode();
 
-    bool chatmode[protocolname_size], rus[protocolname_size],
-	entersends[protocolname_size];
+    bool chatmode[protocolname_size], conv[protocolname_size],
+	entersends[protocolname_size], nonimonline[protocolname_size];
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 	chatmode[pname] = conf.getchatmode(pname);
 	entersends[pname] = conf.getentersends(pname);
-	rus[pname] = conf.getrussian(pname);
+	conv[pname] = conf.getcpconvert(pname);
+	nonimonline[pname] = conf.getnonimonline(pname);
     }
 
     for(hasany = false, pname = icq; pname != protocolname_size && !hasany; (int) pname += 1)
@@ -1128,6 +1174,21 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     #endif
 	t.addleaff(i, 0, 23, _(" Enable emacs bindings in text editor : %s "), stryesno(emacs));
 
+	i = t.addnode(_(" Codepages conversion "));
+
+	for(tmp = "", pname = icq; pname != protocolname_size; (int) pname += 1)
+	if(conv[pname])
+	if(!conf.getourid(pname).empty() || pname == rss)
+	    tmp += conf.getprotocolname(pname) + " ";
+
+	t.addleaff(i, 0, 26, _(" Switch to language preset : %s "), iconvlang->c_str());
+	t.addleaff(i, 0, 27, _(" Convert from : %s "), convertfrom.c_str());
+	t.addleaff(i, 0, 28, _(" Convert to : %s "), convertto.c_str());
+
+	if(hasany) {
+	    t.addleaff(i, 0,  3, _(" For protocols : %s"), tmp.c_str());
+	}
+
 	i = t.addnode(_(" Contact list "));
 	t.addleaff(i, 0, 17, _(" Arrange contacts into groups : %s "), strgroupmode(gmode));
 	t.addleaff(i, 0,  6, _(" Hide offline users : %s "), stryesno(hideoffl));
@@ -1150,21 +1211,16 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 
 	    t.addleaff(i, 0, 25, _(" Enter key sends message for : %s"), tmp.c_str());
 
-	    for(tmp = "", pname = icq; pname != protocolname_size; (int) pname += 1)
-		if(rus[pname])
-		if(!conf.getourid(pname).empty() || pname == rss)
-		    tmp += conf.getprotocolname(pname) + " ";
-
-	    t.addleaff(i, 0,  3, _(" Russian win/koi translation for : %s"), tmp.c_str());
-
 	} else {
 	    t.addleaff(i, 0, 16, _(" Chat messaging mode : %s "), stryesno(chatmode[icq]));
 	    t.addleaff(i, 0, 25, _(" Enter key sends message : %s "), stryesno(entersends[icq]));
-	    t.addleaff(i, 0,  3, _(" Russian win/koi translation : %s "), stryesno(rus[icq]));
 
 	}
 
-	t.addleaff(i, 0, 11, _(" Default charset : %s "), defcharset.c_str());
+	for(tmp = "", pname = icq; pname != protocolname_size; (int) pname += 1)
+	    if(nonimonline[pname]) tmp += conf.getprotocolname(pname) + " ";
+
+	t.addleaff(i, 0, 29, _(" Always online non-IM contacts for : %s"), tmp.c_str());
 
 	i = t.addnode(_(" Communications "));
 	t.addleaff(i, 0, 19, _(" SMTP server : %s "), smtp.c_str());
@@ -1203,9 +1259,9 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 				icqconf::rcdontchange;
 			break;
 		    case 3:
-			if(hasany) selectproto(rus, true); else
+			if(hasany) selectproto(conv, spIMplusRSS); else
 			    for(pname = icq; pname != protocolname_size; (int) pname += 1)
-				rus[pname] = !rus[pname];
+				conv[pname] = !conv[pname];
 			break;
 		    case 4:
 			tmp = inputstr(_("Auto Away period (0 - disable): "), i2str(aaway));
@@ -1220,10 +1276,6 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		    case 8: quote = !quote; break;
 		    case 9: logtimestamps = !logtimestamps; break;
 		    case 10: logonline = !logonline; break;
-		    case 11:
-			tmp = inputstr(_("Default charset: "), defcharset);
-			if(!tmp.empty()) defcharset = tmp;
-			break;
 		    case 13: savepwd = !savepwd; break;
 		    case 14: antispam = !antispam; break;
 		    case 15: mailcheck = !mailcheck; break;
@@ -1272,6 +1324,41 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 			    for(pname = icq; pname != protocolname_size; (int) pname += 1)
 				entersends[pname] = !entersends[pname];
 			break;
+		    case 26:
+			iconvlang++;
+			if(iconvlang == convlanguages.end())
+			    iconvlang = convlanguages.begin();
+
+			switch(iconvlang-convlanguages.begin()) {
+			    case 0:
+				convertto = convertfrom = "";
+				// multichange(conv, false);
+				break;
+
+			    case 1:
+				convertfrom = "cp1251";
+				convertto = "koi8-r";
+				multichange(conv, true);
+				break;
+
+			    case 2:
+				convertfrom = "cp1250";
+				convertto = "iso-8859-2";
+				multichange(conv, true);
+				break;
+			}
+			break;
+		    case 27:
+			convertfrom = inputstr(_("Charset to convert messages from: "), convertfrom);
+			if(input.getlastkey() == '\r') iconvlang = convlanguages.begin();
+			break;
+		    case 28:
+			convertto = inputstr(_("Charset to convert messages to: "), convertto);
+			if(input.getlastkey() == '\r') iconvlang = convlanguages.begin();
+			break;
+		    case 29:
+			selectproto(nonimonline, spnonIM);
+			break;
 		}
 		break;
 	    case 1:
@@ -1285,12 +1372,15 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		conf.setmailcheck(mailcheck);
 		conf.setmakelog(makelog);
 		conf.setaskaway(askaway);
-		conf.setdefcharset(defcharset);
+		conf.setcharsets(convertfrom, convertto);
 
 		for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 		    conf.setchatmode(pname, chatmode[pname]);
 		    conf.setentersends(pname, entersends[pname]);
-		    conf.setrussian(pname, rus[pname]);
+		    conf.setnonimonline(pname, nonimonline[pname]);
+
+		    bool bconv = conv[pname] && (!convertfrom.empty() || !convertto.empty());
+		    conf.setcpconvert(pname, bconv || !hasany);
 		}
 
 		conf.setbidi(bidi);
@@ -1317,7 +1407,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     return success;
 }
 
-void icqface::selectproto(bool prots[], bool irss) {
+void icqface::selectproto(bool prots[], spmode mode) {
     static int saveelem = 0;
     int i, protmax;
     bool r, finished = false;
@@ -1330,15 +1420,29 @@ void icqface::selectproto(bool prots[], bool irss) {
     memcpy(aprots, prots, sizeof(aprots));
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	if(!irss)
-	if(gethook(pname).getCapabs().count(hookcapab::nochat))
-	    continue;
+	if(mode == spnonIM) {
+	    if(pname != infocard)
+	    if(!gethook(pname).getCapabs().count(hookcapab::nochat))
+		continue;
 
-	if(!gethook(pname).enabled())
-	    continue;
+	    if(pname == livejournal)
+		continue;
 
-	if(!conf.getourid(pname).empty() || irss)
-	    tempprots[i++] = pname;
+	} else {
+	    if(mode != spIMplusRSS || pname != rss) {
+		if(gethook(pname).getCapabs().count(hookcapab::nochat))
+		    continue;
+
+		if(conf.getourid(pname).empty())
+		    continue;
+
+	    }
+
+	    if(!gethook(pname).enabled())
+		continue;
+	}
+
+	tempprots[i++] = pname;
     }
 
     protmax = i;
