@@ -1,9 +1,9 @@
 /*
 *
 * centericq user interface class
-* $Id: icqface.cc,v 1.202 2003/11/23 01:09:34 konst Exp $
+* $Id: icqface.cc,v 1.218 2004/03/20 16:49:08 konst Exp $
 *
-* Copyright (C) 2001-2003 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -34,8 +34,6 @@
 #include "irchook.h"
 #include "imexternal.h"
 
-#include <libicq2000/userinfohelpers.h>
-
 const char *stryesno(bool b) {
     return b ? _("yes") : _("no");
 }
@@ -213,7 +211,7 @@ void icqface::showtopbar() {
 	ia = conf.getourid(pname);
 
 	if(!ia.empty()) {
-	    buf += " " + conf.getprotocolname(pname) + "<";
+	    buf += conf.getprotocolname(pname) + "<";
 
 	    buf += imstatus2char[gethook(pname).getstatus()];
 	    buf += ">";
@@ -222,8 +220,8 @@ void icqface::showtopbar() {
 
     attrset(conf.getcolor(cp_status));
     mvhline(0, 0, ' ', COLS);
-    mvprintw(0, 2, _("CENTERICQ %s  UNSENT: %lu"), VERSION, em.getunsentcount());
-    mvprintw(0, COLS-buf.size()-2, "%s", buf.c_str());
+    mvprintw(0, 0, _(" CENTERICQ %s  UNSENT: %lu"), VERSION, em.getunsentcount());
+    mvprintw(0, COLS-buf.size()-1, "%s", buf.c_str());
 }
 
 void icqface::update() {
@@ -246,17 +244,17 @@ int icqface::contextmenu(icqcontact *c) {
     static map<int, string> actnames;
 
     if(actnames.empty()) {
-	actnames[ACT_URL]       = _(" Send an URL            u");
+	actnames[ACT_URL]       = getmenuitem(_("Send an URL"), 25, key_send_url, section_contact);
 	actnames[ACT_SMS]       = _(" Send an SMS");
-	actnames[ACT_CONTACT]   = _(" Send contacts          c");
+	actnames[ACT_CONTACT]   = getmenuitem(_("Send contacts"), 25, key_send_contact, section_contact);
 	actnames[ACT_AUTH]      = _(" Request authorization");
-	actnames[ACT_EDITUSER]  = _(" Edit details           e");
-	actnames[ACT_FETCHAWAY] = _(" Fetch away message     f");
-	actnames[ACT_ADD]       = _(" Add to list            a");
-	actnames[ACT_RENAME]    = _(" Rename contact         r");
+	actnames[ACT_EDITUSER]  = getmenuitem(_("Edit details"), 25, key_edit, section_contact);
+	actnames[ACT_FETCHAWAY] = getmenuitem(_("Fetch away message"), 25, key_fetch_away_message, section_contact);
+	actnames[ACT_ADD]       = getmenuitem(_("Add to list"), 25, key_add, section_contact);
+	actnames[ACT_RENAME]    = getmenuitem(_("Rename contact"), 25, key_rename, section_contact);
 	actnames[ACT_GROUPMOVE] = _(" Move to group..");
 	actnames[ACT_PING]      = _(" Ping");
-	actnames[ACT_VERSION]   = _(" Fetch version info     v");
+	actnames[ACT_VERSION]   = getmenuitem(_("Fetch version info"), 25, key_rename, section_contact);
 	actnames[ACT_FILE]      = _(" Send file(s)");
 	actnames[ACT_CONFER]    = _(" Invite to conference..");
     }
@@ -266,45 +264,34 @@ int icqface::contextmenu(icqcontact *c) {
 	    sizeWArea.y1+6, conf.getcolor(cp_main_text)));
 
 	actnames[ACT_MSG]       = _(" Send a channel message   enter");
-	actnames[ACT_HISTORY]   = _(" Channel chat history         h");
+	actnames[ACT_HISTORY]   = getmenuitem(_("Channel chat history"), 25, key_history, section_contact);
 	actnames[ACT_REMOVE]    = _(" Remove channel             del");
 	actnames[ACT_JOIN]      = _(" Join channel");
 	actnames[ACT_LEAVE]     = _(" Leave channel");
-	actnames[ACT_INFO]      = _(" List nicknames               ?");
+	actnames[ACT_INFO]      = getmenuitem(_("List nichnames"), 25, key_info, section_contact);
 
 	if(lst.inlist(c, csignore))
-	    actnames[ACT_IGNORE]    = _(" UnBlock channel messages     i");
+	    actnames[ACT_IGNORE]    = getmenuitem(_("UnBlock channel messages"), 25, key_ignore, section_contact);
 	else
-	    actnames[ACT_IGNORE]    = _(" Block channel messages       i");
+	    actnames[ACT_IGNORE]    = getmenuitem(_("Block channel messages"), 25, key_ignore, section_contact);
 
     } else {
 	m.setwindow(textwindow(sizeWArea.x1, sizeWArea.y1, sizeWArea.x1+27,
 	    sizeWArea.y1+6, conf.getcolor(cp_main_text)));
 
-	if(c->getdesc().pname != rss) {
-	    actnames[ACT_MSG]       = _(" Send a message     enter");
-	    actnames[ACT_HISTORY]   = _(" Events history         h");
-	    actnames[ACT_REMOVE]    = _(" Remove user          del");
-	    actnames[ACT_INFO]      = _(" User's details         ?");
-	    actnames[ACT_EXTERN]    = _(" External actions..    f6");
-
-	    if(lst.inlist(c, csignore))
-		actnames[ACT_IGNORE]    = _(" Unset ignore user      i");
+	if(c->getdesc().pname != rss)
+	    actnames[ACT_MSG]       = getmenuitem(_("Send a message"), 25, key_compose, section_contact);
+	actnames[ACT_HISTORY]   = getmenuitem(_("Event history"), 25, key_history, section_contact);
+	actnames[ACT_REMOVE]    = getmenuitem(_("Remove user"), 25, key_remove, section_contact);
+	actnames[ACT_INFO]      = getmenuitem(_("User's details"), 25, key_info, section_contact);
+	actnames[ACT_EXTERN]    = getmenuitem(_("External actions.."), 25, key_contact_external_action, section_contact);
+	if(lst.inlist(c, csignore))
+	    actnames[ACT_IGNORE]    = getmenuitem(_("Unset ignore user"), 25, key_ignore, section_contact);
 	    else
-		actnames[ACT_IGNORE]    = _(" Ignore user            i");
+		actnames[ACT_IGNORE]    = getmenuitem(_("Ignore user"), 25, key_ignore, section_contact);
+	if(c->getdesc().pname == rss)
+	    actnames[ACT_PING]      = getmenuitem(_("Force check"), 25, key_rss_check, section_contact);
 
-	} else {
-	    actnames[ACT_HISTORY]   = _(" Events history         h");
-	    actnames[ACT_REMOVE]    = _(" Remove feed          del");
-	    actnames[ACT_INFO]      = _(" Feed information       ?");
-	    actnames[ACT_EXTERN]    = _(" External actions..    f6");
-	    actnames[ACT_PING]      = _(" Force check");
-
-	    if(lst.inlist(c, csignore))
-		actnames[ACT_IGNORE]    = _(" Unset ignore feed      i");
-	    else
-		actnames[ACT_IGNORE]    = _(" Ignore feed            i");
-	}
     }
 
     cont = c->getdesc();
@@ -401,8 +388,8 @@ int icqface::generalmenu() {
     m.setwindow(textwindow(sizeWArea.x1, sizeWArea.y1, sizeWArea.x1+40, sizeWArea.y1+11, conf.getcolor(cp_main_text)));
 
     m.idle = &menuidle;
-    m.additem(0, ACT_STATUS,    _(" Change status                       s"));
-    m.additem(0, ACT_QUICKFIND, _(" Go to contact..                 alt-s"));
+    m.additem(0, ACT_STATUS,    getmenuitem(_("Change status"), 38, key_change_status, section_contact));
+    m.additem(0, ACT_QUICKFIND, getmenuitem(_("Go to contact.."), 38, key_quickfind, section_contact));
     m.additem(0, ACT_DETAILS,   _(" Accounts.."));
     m.additem(0, ACT_CONF,      _(" CenterICQ config options"));
     m.additem(0, ACT_TRANSFERS, _(" File transfers monitor"));
@@ -417,12 +404,14 @@ int icqface::generalmenu() {
     m.additem(0, ACT_INVISLIST,     _(" View/edit invisible list"));
     m.additem(0, ACT_VISIBLELIST,   _(" View/edit visible list"));
     m.addline();
-    m.additem(0, ACT_HIDEOFFLINE, conf.gethideoffline() ?
-	_(" Show offline users                 F5") :
-	_(" Hide offline users                 F5"));
+    if(conf.gethideoffline())
+	m.additem(0, ACT_HIDEOFFLINE, getmenuitem(_("Show offline users"), 38, key_hide_offline, section_contact));
+    else
+	m.additem(0, ACT_HIDEOFFLINE, getmenuitem(_("Hide offline users"), 38, key_hide_offline, section_contact));
 
     if(conf.getgroupmode() != icqconf::nogroups) {
 	m.additem(0, ACT_ORG_GROUPS, _(" Organize contact groups"));
+	m.additem(0, ACT_MASS_MOVE, _(" Mass group move.."));
     }
 
     m.setpos(lastitem);
@@ -830,7 +819,7 @@ bool icqface::findresults(const imsearchparams &sp, bool fauto) {
 		case 2:
 		    if(r) {
 			c = (icqcontact *) db.getmenu()->getref(r-1);
-			if(c) cicq.addcontact(c->getdesc());
+			if(c) cicq.addcontact(c->getdesc(), c->getbasicinfo().requiresauth);
 		    }
 		    break;
 
@@ -971,9 +960,9 @@ void icqface::infogeneral(dialogbox &db, icqcontact *c) {
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+9, conf.getcolor(cp_main_text), mi.strbirthdate());
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+10, conf.getcolor(cp_main_text), mi.age ? i2str(mi.age) : "");
 
-    if(mi.lang1) commacat(langs, ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang1), langs.empty());
-    if(mi.lang2) commacat(langs, ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang2), langs.empty());
-    if(mi.lang3) commacat(langs, ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang3), langs.empty());
+    if(mi.lang1) commacat(langs, abstracthook::getLanguageIDtoString(mi.lang1), langs.empty());
+    if(mi.lang2) commacat(langs, abstracthook::getLanguageIDtoString(mi.lang2), langs.empty());
+    if(mi.lang3) commacat(langs, abstracthook::getLanguageIDtoString(mi.lang3), langs.empty());
     commaform(langs);
 
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+12, conf.getcolor(cp_main_text), langs);
@@ -1014,7 +1003,7 @@ void icqface::infohome(dialogbox &db, icqcontact *c) {
 
     if(bi.country) {
 	if(bi.city.size()) bi.city += ", ";
-	bi.city += ICQ2000::UserInfoHelpers::getCountryIDtoString(bi.country);
+	bi.city += abstracthook::getCountryIDtoString(bi.country);
     }
 
     x += 10;
@@ -1059,7 +1048,7 @@ void icqface::infowork(dialogbox &db, icqcontact *c) {
 
     if(wi.country) {
 	if(wi.city.size()) wi.city += ", ";
-	wi.city += ICQ2000::UserInfoHelpers::getCountryIDtoString(wi.country);
+	wi.city += abstracthook::getCountryIDtoString(wi.country);
     }
 
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+2, conf.getcolor(cp_main_text), wi.street);
@@ -1186,7 +1175,10 @@ void icqface::userinfo(const imcontact &cinfo, const imcontact &realinfo) {
     saveworkarea();
     clearworkarea();
 
-    status(_("F2 to URLs, F6 external actions, ESC close"));
+    status(_("%s to URLs, %s external actions, %s close"),
+	getstatkey(key_show_urls, section_info).c_str(),
+	getstatkey(key_user_external_action, section_info).c_str(),
+	getstatkey(key_quit, section_editor).c_str());
 
     db.setwindow(new textwindow(sizeWArea.x1+1, sizeWArea.y1+2, sizeWArea.x2,
 	sizeWArea.y2, conf.getcolor(cp_main_text), TW_NOBORDER));
@@ -1321,6 +1313,7 @@ void icqface::makeprotocolmenu(verticalmenu &m) {
 	_(" [jab] Jabber"),
 	"",
 	_(" [lj] LiveJournal"),
+	_(" [gg] Gadu-Gadu"),
 	""
     };
 
@@ -1889,6 +1882,16 @@ void icqface::status(const string &text) {
     kwriteatf(0, LINES-1, conf.getcolor(cp_status), "%s", t.c_str());
 }
 
+void icqface::status(const char *fmt, ...) {
+    va_list ap;
+    char buf[1024];
+
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    status((string) buf);
+    va_end(ap);
+}
+
 void icqface::blockmainscreen() {
     mainscreenblock = true;
 }
@@ -1907,7 +1910,8 @@ void icqface::quickfind(verticalmenu *multi) {
     icqcontact *c;
     verticalmenu *cm = (multi ? multi : &mcontacts->menu);
 
-    status(_("QuickSearch: type to find, Alt-S find again, Enter finish"));
+    status(_("QuickSearch: type to find, %s find again, Enter finish"),
+	getstatkey(key_send_message, section_editor).c_str());
 
     if(multi) {
 	lx = sizeWArea.x1+2;
@@ -2087,11 +2091,18 @@ void icqface::userinfoexternal(const imcontact &ic) {
 }
 
 void icqface::showeventbottom(const imcontact &ic) {
-    if(ischannel(ic)) {
-	status(_("^X send, ^P multi, ^O history, F2 URLs, F9 expand, Alt-? members, ESC close"));
-    } else {
-	status(_("^X send, ^P multi, ^O history, F2 URLs, F9 expand, Alt-? details, ESC cancel"));
-    }
+    const char *text = ischannel(ic) ?
+	_("%s send, %s multi, %s history, %s URLs, %s expand, %s members, %s close") :
+	_("%s send, %s multi, %s history, %s URLs, %s expand, %s details, %s cancel");
+
+    status(text,
+	getstatkey(key_send_message, section_editor).c_str(),
+	getstatkey(key_multiple_recipients, section_editor).c_str(),
+	getstatkey(key_history, section_editor).c_str(),
+	getstatkey(key_show_urls, section_editor).c_str(),
+	getstatkey(key_fullscreen, section_editor).c_str(),
+	getstatkey(key_info, section_editor).c_str(),
+	getstatkey(key_quit, section_editor).c_str());
 }
 
 bool icqface::eventedit(imevent &ev) {
@@ -2333,11 +2344,29 @@ bool icqface::eventedit(imevent &ev) {
     return r;
 }
 
+string icqface::extracttime(const imevent &ev) {
+    string ds1, ds2, r;
+    int tdiff;
+    time_t t, ts;
+    char buf[64];
+
+    ds1 = time2str(&(t = ev.gettimestamp()), "DD.MM hh:mm", buf);
+    ds2 = time2str(&(ts = ev.getsenttimestamp()), "DD.MM hh:mm", buf);
+
+    r = ds1 + " ";
+    tdiff = ts-t;
+    if(tdiff < 0) tdiff *= -1;
+
+    if(tdiff > 5)
+    if(ds1 != ds2) r += (string) "[" + ds2 + "] ";
+
+    return r;
+}
+
 void icqface::renderchathistory() {
     int count, chatmargin;
     string text;
-    char buf[64];
-    time_t t, lastread;
+    time_t lastread;
     struct stat st;
 
     vector<imevent *> events;
@@ -2398,9 +2427,9 @@ void icqface::renderchathistory() {
 	}
 
 	if(count < chatlines) {
-	    text = (string) time2str(&(t = events.back()->gettimestamp()), "DD.MM hh:mm", buf) + " ";
+	    text = extracttime(*events.back());
 	    text += events.back()->gettext();
-	    chatlastread = t-1;
+	    chatlastread = events.back()->gettimestamp()-1;
 
 	    h = histentry();
 	    h.first = events.back()->getdirection();
@@ -2532,7 +2561,7 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 	    actions.push_back(forward);
 
 	    if(ev->getdirection() == imevent::incoming)
-	    if(ev->getcontact().pname != rss || islivejournal(ev->getcontact()))
+	    if(ev->getcontact().pname != rss /*|| islivejournal(ev->getcontact())*/)
 		actions.push_back(reply);
 
 	} else if(ev->gettype() == imevent::url) {
@@ -2604,7 +2633,7 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 	title_event.c_str(), streventname(ev->gettype()), ev->getcontact().totext().c_str());
 
     mainw.writef(sizeWArea.x1+2, sizeWArea.y1+1, conf.getcolor(cp_main_highlight),
-	title_timestamp.c_str(), strdateandtime(ev->gettimestamp()).c_str());
+	title_timestamp.c_str(), extracttime(*ev).c_str());
 
     db.addautokeys();
 
@@ -2628,8 +2657,11 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 	db.setbrowser(new textbrowser(conf.getcolor(cp_main_text)));
 	db.getbrowser()->setbuf(text);
 	extracturls(text);
-	status(_("F2 to URLs, F9 to full-screenize, ESC close"));
 
+	status(_("%s to URLs, %s to full-screenize, %s close"),
+	    getstatkey(key_show_urls, section_info).c_str(),
+	    getstatkey(key_fullscreen, section_editor).c_str(),
+	    getstatkey(key_quit, section_editor).c_str());
     }
 
     db.redraw();
@@ -2663,14 +2695,14 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 
 void icqface::fullscreenize(const imevent *ev) {
     textwindow w(0, 1, COLS, LINES-1, conf.getcolor(cp_main_text), TW_NOBORDER);
-    w.open();
+    textbrowser tb(0, 4, COLS, LINES-1, conf.getcolor(cp_main_text));
 
     int k;
     char buf[512], *fmt = 0;
     vector<string> lines;
     vector<string>::const_iterator il;
 
-    breakintolines(ev->gettext(), lines, COLS);
+    w.open();
 
     switch(ev->getdirection()) {
 	case imevent::incoming: fmt = _("%s from %s, received on %s"); break;
@@ -2683,17 +2715,17 @@ void icqface::fullscreenize(const imevent *ev) {
 
     w.write(0, 1, conf.getcolor(cp_main_highlight), buf);
 
-    for(il = lines.begin(); il != lines.end() && (il-lines.begin() < LINES-1); ++il)
-	w.write(0, il-lines.begin()+3, *il);
+    tb.setbuf(ev->gettext());
 
     blockmainscreen();
-    status(_("ESC or F9 to close"));
 
-    do {
-	cicq.idle();
-	k = getkey();
+    status(_("%s or %s to close, Up/Down and PgUp/PgDn to scroll"),
+	getstatkey(key_quit, section_editor).c_str(),
+	getstatkey(key_fullscreen, section_editor).c_str());
 
-    } while(k != KEY_ESC && k != KEY_F(9));
+    tb.idle = &textbrowseridle;
+    tb.otherkeys = &fullscreenkeys;
+    tb.open();
 
     unblockmainscreen();
     status("@");
@@ -2704,7 +2736,7 @@ void icqface::fullscreenize(const imevent *ev) {
 void icqface::histmake(const vector<imevent *> &hist) {
     vector<imevent *>::const_reverse_iterator i;
     string text;
-    time_t t;
+    time_t t, ts;
     char buf[64];
     int color;
 
@@ -2716,7 +2748,11 @@ void icqface::histmake(const vector<imevent *> &hist) {
 
 	color = 0;
 
-	text = (string) + " " + time2str(&(t = ev.gettimestamp()), "DD.MM hh:mm", buf) + " ";
+	t = ev.gettimestamp();
+	ts = ev.getsenttimestamp();
+	text = (string) + " " + time2str(&t, "DD.MM hh:mm", buf) + " ";
+	if ((t - ts) > 0) 
+	    text += (string) + "[" + time2str(&ts, "DD.MM hh:mm", buf) + "] ";
 	text += ev.gettext();
 
 	if(ev.getdirection() == imevent::incoming) {
@@ -2772,7 +2808,10 @@ bool icqface::histexec(imevent *&im) {
 	    _("History for %s, %d events total"),
 	    im->getcontact().totext().c_str(), mhist.getcount());
 
-	status(_("/ search, N again, ESC cancel"));
+	status(_("%s search, %s again, %s cancel"),
+	    getstatkey(key_search, section_history).c_str(),
+	    getstatkey(key_search_again, section_history).c_str(),
+	    getstatkey(key_quit, section_editor).c_str());
 
 	while(!fin) {
 	    if(db.open(k)) {
@@ -2866,6 +2905,112 @@ void icqface::transferidle(dialogbox &b) {
     }
 }
 
+string icqface::getstatkey(int key, int section) const {
+    int i = 2;
+    string key2;
+    string keys = action2key(key, section);
+
+    while((key2 = action2key(key, section, i)) != "") {
+	keys += "/" + key2;
+	i++;
+    }
+
+    return keys;
+}
+
+string icqface::getmenuitem(string mtext, int width, int key, int section) {
+    int i = 2;
+    string text = " " + mtext + " ";
+    string keyname = action2key(key, section);
+    string key2;
+    int s = keyname.size()+text.size();
+
+    if (s > width)
+	return text;
+
+    while ((key2 = action2key(key, section, i)) != "" && (s += key2.size()+1) < width) {
+	keyname += "/" + key2;
+	i++;
+    }
+
+    for (s = keyname.size()+text.size(); s < width; s++)
+	text += " ";
+
+    text += keyname;
+
+    return text;
+}
+
+int icqface::key2action(int k, int s) {
+    for(vector<icqconf::keybinding>::size_type i = 0; i < icqconf::keys.size(); i++)
+	if(k == icqconf::keys[i].key && s == icqconf::keys[i].section)
+	    return icqconf::keys[i].action;
+
+    return -1;
+}
+
+string icqface::action2key(int a, int s, int n) const {
+    string key;
+#ifdef HAVE_SSTREAM
+    std::ostringstream o;
+#else
+    std::ostrstream o;
+#endif
+    vector<icqconf::keybinding>::size_type i = 0;
+
+    for( ; i < icqconf::keys.size(); i++)
+	if(a == icqconf::keys[i].action && s == icqconf::keys[i].section) {
+	    if (n <= 1)
+		break;
+	    n--;
+	}
+
+    if (i == icqconf::keys.size())
+	return "";
+    else if(icqconf::keys[i].key == '\r')
+	return "enter";
+    else if(icqconf::keys[i].key == ' ')
+	return "space";
+    else if(icqconf::keys[i].key == '/')
+	return "\'/\'";
+    else if(icqconf::keys[i].key == ALT('\r'))
+	return "alt-enter";
+    else if(icqconf::keys[i].key == CTRL(' '))
+	return "^space";
+    else if(icqconf::keys[i].key == ALT(' '))
+	return "alt-space";
+    else if(icqconf::keys[i].key == 339)
+	return "pageup";
+    else if(icqconf::keys[i].key == 338)
+	return "pagedown";
+    else if(icqconf::keys[i].key == 9)
+	return "tab";
+    else if(icqconf::keys[i].key == 27)
+	return "esc";
+    else if(icqconf::keys[i].key == 331)
+	return "insert";
+    else if(icqconf::keys[i].key == KEY_DC)
+	return "del";
+    else if(icqconf::keys[i].alt && icqconf::keys[i].ctrl)
+	o << "F" << (icqconf::keys[i].key-KEY_F0);
+    else if(icqconf::keys[i].alt)
+	o << "alt-" << (char)icqconf::keys[i].key;
+    else if(icqconf::keys[i].ctrl)
+	o << "^" << (char)(icqconf::keys[i].key+64);
+    else
+	o << (char)icqconf::keys[i].key;
+
+    string ret;
+#ifdef HAVE_SSTREAM
+    ret = o.str();
+#else
+    ret = string(o.str(), o.pcount());
+    o.freeze(false);
+#endif
+
+    return ret;
+}
+
 int icqface::contactskeys(verticalmenu &m, int k) {
     icqcontact *c = NULL;
 
@@ -2882,108 +3027,88 @@ int icqface::contactskeys(verticalmenu &m, int k) {
 	capab = gethook(c->getdesc().pname).getCapabs();
     }
 
-    switch(k) {
-	case '?':
+    switch(face.key2action(k, section_contact)) {
+	case key_info:
 	    face.extk = ACT_INFO;
 	    break;
 
-	case KEY_DC:
+	case key_remove:
 	    face.extk = ACT_REMOVE;
 	    break;
 
-	case 'q':
-	case 'Q':
+	case key_quit:
 	    face.extk = ACT_QUIT;
 	    break;
 
-	case 'u':
-	case 'U':
+	case key_send_url:
 	    if(capab.count(hookcapab::urls))
 		face.extk = ACT_URL;
 	    break;
 
-	case 's':
-	case 'S':
-	case KEY_F(3):
+	case key_change_status:
 	    face.extk = ACT_STATUS;
 	    break;
 
-	case 'h':
-	case 'H':
+	case key_history:
 	    face.extk = ACT_HISTORY;
 	    break;
 
-	case 'a':
-	case 'A': face.extk = ACT_ADD; break;
+	case key_add: face.extk = ACT_ADD; break;
 
-	case 'c':
-	case 'C':
+	case key_send_contact:
 	    if(capab.count(hookcapab::contacts))
 		face.extk = ACT_CONTACT;
 	    break;
+	case key_rss_check:
+	    if(c && c->getdesc().pname == rss)
+		face.extk = ACT_PING;
+	    break;
 
-	case 'f':
-	case 'F':
+	case key_fetch_away_message:
 	    if(capab.count(hookcapab::fetchaway))
 		face.extk = ACT_FETCHAWAY;
 	    break;
 
-	case 'm':
-	case 'M':
-	case KEY_F(2):
+	case key_user_menu:
 	    face.extk = ACT_MENU;
 	    break;
 
-	case 'g':
-	case 'G':
-	case KEY_F(4):
+	case key_general_menu:
 	    face.extk = ACT_GMENU;
 	    break;
 
-	case KEY_F(5):
+	case key_hide_offline:
 	    face.extk = ACT_HIDEOFFLINE;
 	    break;
 
-	case KEY_F(6):
+	case key_contact_external_action:
 	    if(!ischannel(c) && c)
 		face.extk = ACT_EXTERN;
 	    break;
 
-	case 'r':
-	case 'R':
+	case key_rename:
 	    if(!ischannel(c) && c)
 		face.extk = ACT_RENAME;
 	    break;
 
-	case 'v':
-	case 'V':
+	case key_version:
 	    if(capab.count(hookcapab::version))
 		face.extk = ACT_VERSION;
 	    break;
 
-	case 'e':
-	case 'E':
+	case key_edit:
 	    if(!ischannel(c) && c)
 		face.extk = ACT_EDITUSER;
 	    break;
 
-	case 'i':
-	case 'I':
+	case key_ignore:
 	    if(c) face.extk = ACT_IGNORE;
 	    break;
 
-	case ALT('s'):
-	case '/': face.extk = ACT_QUICKFIND; break;
+	case key_quickfind: face.extk = ACT_QUICKFIND; break;
     }
 
-    if(k && face.extk && (strchr("?rRqQsShHmMuUgGaAfFcCeEiIvV/", k)
-	|| (k == KEY_DC)
-	|| (k == KEY_F(2))
-	|| (k == KEY_F(3))
-	|| (k == KEY_F(4))
-	|| (k == KEY_F(5))
-	|| (k == KEY_F(6))
-	|| (k == ALT('s')))) {
+    if(face.extk && face.key2action(k, section_contact) != -1) {
 	return m.getpos()+1;
     } else {
 	return -1;
@@ -3007,13 +3132,10 @@ int icqface::multiplekeys(verticalmenu &m, int k) {
 int icqface::historykeys(dialogbox &db, int k) {
     static string sub;
 
-    switch(k) {
-	case 's':
-	case 'S':
-	case '/':
+    switch(face.key2action(k, section_history)) {
+	case key_search:
 	    return -2;
-	case 'n':
-	case 'N':
+	case key_search_again:
 	    return -3;
     }
 
@@ -3023,31 +3145,31 @@ int icqface::historykeys(dialogbox &db, int k) {
 int icqface::editmsgkeys(texteditor &e, int k) {
     char *p;
 
-    switch(k) {
+    switch(face.key2action(k, section_editor)) {
 	case '\r':
 	    if(!conf.getentersends(face.passinfo.pname))
 		break;
-	case CTRL('x'):
+	case key_send_message:
 	    p = e.save("");
 	    face.editdone = strlen(p);
 	    delete p;
 	    if(face.editdone) return -1; else break;
-	case CTRL('p'):
+	case key_multiple_recipients:
 	    face.multicontacts("");
 	    break;
-	case CTRL('o'):
+	case key_history:
 	    cicq.history(face.passinfo);
 	    break;
-	case ALT('?'):
+	case key_info:
 	    cicq.userinfo(face.passinfo);
 	    break;
-	case KEY_F(2):
+	case key_show_urls:
 	    face.showextractedurls(); break;
-	case KEY_F(9):
+	case key_fullscreen:
 	    if(face.passevent)
 		face.fullscreenize(face.passevent);
 	    break;
-	case 27:
+	case key_quit:
 	    return -1;
     }
 
@@ -3055,20 +3177,20 @@ int icqface::editmsgkeys(texteditor &e, int k) {
 }
 
 int icqface::userinfokeys(dialogbox &db, int k) {
-    switch(k) {
-	case KEY_F(2): face.showextractedurls(); break;
-	case KEY_F(6): face.userinfoexternal(face.passinfo); break;
+    switch(face.key2action(k, section_info)) {
+	case key_show_urls: face.showextractedurls(); break;
+	case key_user_external_action: face.userinfoexternal(face.passinfo); break;
     }
 
     return -1;
 }
 
 int icqface::eventviewkeys(dialogbox &db, int k) {
-    switch(k) {
-	case KEY_F(2):
+    switch(face.key2action(k, section_history)) {
+	case key_show_urls:
 	    face.showextractedurls();
 	    break;
-	case KEY_F(9):
+	case key_fullscreen:
 	    face.fullscreenize(face.passevent);
 	    break;
     }
@@ -3077,11 +3199,8 @@ int icqface::eventviewkeys(dialogbox &db, int k) {
 }
 
 int icqface::findreskeys(dialogbox &db, int k) {
-    switch(k) {
-	case ALT('s'):
-	case '/':
-	    return -3;
-    }
+    if (face.key2action(k, section_history) == key_search)
+	return -3;
 
     return -1;
 }
@@ -3094,6 +3213,15 @@ int icqface::statuskeys(verticalmenu &m, int k) {
     return -1;
 }
 
+int icqface::fullscreenkeys(textbrowser &m, int k) {
+    switch(k) {
+	case KEY_F(9):
+	    return 0;
+    }
+
+    return -1;
+}
+
 void icqface::editidle(texteditor &e) {
     cicq.idle();
 }
