@@ -1,9 +1,9 @@
 /*
 *
 * centericq protocol specific user interface related routines
-* $Id: imcontroller.cc,v 1.48 2003/11/05 09:07:41 konst Exp $
+* $Id: imcontroller.cc,v 1.52 2004/02/20 20:48:47 konst Exp $
 *
-* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "aimhook.h"
 #include "irchook.h"
 #include "jabberhook.h"
+#include "msnhook.h"
 #include "icqcontacts.h"
 #include "eventmanager.h"
 
@@ -86,8 +87,10 @@ bool imcontroller::regdialog(protocolname pname) {
 	t.addleaff(i, 0, 2, _(" Password to set : %s "), string(rpasswd.size(), '*').c_str());
 	t.addleaff(i, 0, 3, _(" Check the password : %s "), string(pcheck.size(), '*').c_str());
 
-	i = db.gettree()->addnode(_(" Details "));
-	t.addleaff(i, 0, 8, _(" Server to use : %s "), rserver.c_str());
+	if(pname != gadu) {
+	    i = db.gettree()->addnode(_(" Details "));
+	    t.addleaff(i, 0, 8, _(" Server to use : %s "), rserver.c_str());
+	}
 
 	finished = !db.open(n, b, (void **) &i);
 
@@ -110,6 +113,8 @@ bool imcontroller::regdialog(protocolname pname) {
 		    face.status(_("Passwords do not match"));
 		} else if(rpasswd.empty()) {
 		    face.status(_("Password must be entered"));
+		} else if(pname == icq && rpasswd.size() < 6) {
+		    face.status(_("Password must be at least 6 chars long"));
 		} else {
 		    finished = success = true;
 		}
@@ -122,22 +127,23 @@ bool imcontroller::regdialog(protocolname pname) {
     return success;
 }
 
-bool imcontroller::icqregistration(icqconf::imaccount &account) {
+bool imcontroller::uinreg(icqconf::imaccount &account) {
     bool fin, success;
+    abstracthook &hook = gethook(account.pname);
 
     if(success = regdialog(account.pname)) {
-	unlink((conf.getdirname() + "icq-infoset").c_str());
+	unlink((conf.getdirname() + conf.getprotocolname(account.pname) + "-infoset").c_str());
 	face.progress.show(_(" Registration progress "));
 
 	for(fin = false; !fin; ) {
 	    face.progress.log(_("Connecting to the server %s ..."), rserver.c_str());
 
-	    if(ihook.regconnect(rserver)) {
+	    if(hook.regconnect(rserver)) {
 		while(!fin) {
 		    face.progress.log(_("Sending request"));
 
 		    while(!ruin && !fin) {
-			fin = ihook.regattempt(ruin, rpasswd);
+			fin = hook.regattempt(ruin, rpasswd);
 
 			if(!fin) {
 			    if(fin = (face.ask(_("Timed out waiting for a new uin. Retry?"),
@@ -155,7 +161,7 @@ bool imcontroller::icqregistration(icqconf::imaccount &account) {
 			account.password = rpasswd;
 
 			conf.checkdir();
-			ofstream f((conf.getdirname() + "icq-infoset").c_str());
+			ofstream f((conf.getdirname() + conf.getprotocolname(account.pname) + "-infoset").c_str());
 			if(f.is_open()) {
 			    f << rnick << endl <<
 				remail << endl <<
@@ -169,7 +175,7 @@ bool imcontroller::icqregistration(icqconf::imaccount &account) {
 		face.progress.log(_("Disconnected"));
 		sleep(2);
 	    } else {
-		fin = (face.ask(_("Unable to connect to the icq server. Retry?"),
+		fin = (face.ask(_("Unable to connect to icq server. Retry?"),
 		    ASK_YES | ASK_NO) == ASK_NO);
 	    }
 	}
@@ -298,77 +304,15 @@ void imcontroller::jabberupdateprofile() {
 #endif
 }
 
-void imcontroller::icqsynclist() {
-    bool fin;
-    int synchronized, attempt = 1;
-    string msg;
-    vector<icqcontact *> tobestored;
-
-    face.progress.show(_(" Contact list synchronization "));
-    face.progress.log(_("Starting the synchronization process"));
-
-    for(fin = false; !fin; ) {
-	ihook.getsyncstatus(synchronized, tobestored);
-	face.progress.log(_("Attempt #%d"), attempt);
-	face.progress.log(_(".. %d already synchronized, %d contacts to be stored"), synchronized, tobestored.size());
-
-	msg = "";
-	if(tobestored.empty()) msg = _("Finished");
-	else if(!ihook.logged()) msg = _("Disconnected");
-	else if(attempt > 10) msg = _("Too many attempts, gave up");
-
-	if(!msg.empty()) {
-	    face.progress.log("%s", msg.c_str());
-	    fin = true;
-	    sleep(2);
-	}
-
-	if(!fin) {
-	    ihook.synclist();
-	}
-
-	attempt++;
-    }
-
-    face.progress.hide();
-}
-
-void imcontroller::synclist(protocolname pname) {
-    vector<icqcontact *> tosync = gethook(pname).getneedsync();
-    vector<icqcontact *>::iterator it;
-
-    face.progress.show(_(" Uploading buddies "));
-
-    if(tosync.empty()) {
-	face.progress.log(_("No contacts to be synchronized"));
-	face.progress.log(_(".. all of them are already stored server-side"));
-
-    } else {
-	char buf[512];
-	sprintf(buf, _("%d contacts are do be stored server-side. Add them now?"), tosync.size());
-
-	if(face.ask(buf, ASK_YES | ASK_NO, ASK_YES) == ASK_YES) {
-
-	    for(it = tosync.begin(); it != tosync.end(); ++it) {
-		face.progress.log(_(".. adding %s"), (*it)->getdesc().nickname.c_str());
-		gethook(pname).sendnewuser((*it)->getdesc());
-	    }
-
-	    face.progress.log(_("Finished"));
-	} else {
-	    face.progress.log(_("Cancelled"));
-	}
-
-    }
-
-    sleep(2);
-    face.progress.hide();
-}
-
 void imcontroller::registration(icqconf::imaccount &account) {
     switch(account.pname) {
-	case icq: icqregistration(account); break;
-	case jabber: jabberregistration(account); break;
+	case icq:
+//        case gadu:
+	    uinreg(account);
+	    break;
+	case jabber:
+	    jabberregistration(account);
+	    break;
 	default:
 	    face.status("[" + conf.getprotocolname(account.pname) + "] " +
 		_("registration is not supported"));
@@ -385,18 +329,6 @@ void imcontroller::updateinfo(icqconf::imaccount &account) {
     }
 }
 
-void imcontroller::synclist(icqconf::imaccount &account) {
-    switch(account.pname) {
-	case icq:
-	    icqsynclist();
-	    break;
-
-	default:
-	    synclist(account.pname);
-	    break;
-    }
-}
-
 // ----------------------------------------------------------------------------
 
 void imsearchparams::save(const string &prname) const {
@@ -431,15 +363,15 @@ void imsearchparams::save(const string &prname) const {
 
     if(of.is_open()) {
 	of << prname << "\t"
-	    << dec << (int) pname << "\t"
+	    << (int) pname << "\t"
 	    << (onlineonly ? "1" : "0") << "\t"
-	    << dec << uin << "\t"
-	    << dec << minage << "\t"
-	    << dec << maxage << "\t"
-	    << dec << country << "\t"
-	    << dec << language << "\t"
-	    << dec << (int) agerange << "\t"
-	    << dec << (int) gender << "\t"
+	    << uin << "\t"
+	    << minage << "\t"
+	    << maxage << "\t"
+	    << country << "\t"
+	    << language << "\t"
+	    << (int) agerange << "\t"
+	    << (int) gender << "\t"
 	    << firstname << "\t"
 	    << lastname << "\t"
 	    << nick << "\t"
@@ -454,7 +386,8 @@ void imsearchparams::save(const string &prname) const {
 	    << (reverse ? "1" : "0") << "\t"
 	    << service << "\t"
 	    << (photo ? "1" : "0") << "\t"
-	    << kwords;
+	    << kwords << "\t"
+	    << randomgroup;
 
 	vector<pair<string, string> >::const_iterator
 	    ifp = flexparams.begin();
@@ -515,6 +448,7 @@ bool imsearchparams::load(const string &prname) {
 		service = getfield(buf);
 		photo = getfield(buf) == "1";
 		kwords = getfield(buf);
+		randomgroup = strtoul(getfield(buf).c_str(), 0, 0);
 
 		while(!buf.empty())
 		    if(!(flexparam = getfield(buf)).empty())
