@@ -11,7 +11,7 @@ struct imsearchparams {
 	uin = checkfrequency = 0;
 	minage = maxage = country = language = randomgroup = 0;
 	gender = genderUnspec;
-	agerange = ICQ2000::range_NoRange;
+	agerange = ICQ2000::RANGE_NORANGE;
     };
 
     imsearchparams(const string &name) {
@@ -40,12 +40,10 @@ class imcontroller {
 	unsigned int ruin;
 	string rnick, rfname, rlname, remail, rpasswd, rserver;
 
-	void synclist(protocolname pname);
-
     protected:
 	bool regdialog(protocolname pname);
 
-	bool icqregistration(icqconf::imaccount &account);
+	bool uinreg(icqconf::imaccount &account);
 	bool jabberregistration(icqconf::imaccount &account);
 
 	void icqupdatedetails();
@@ -53,16 +51,12 @@ class imcontroller {
 	void msnupdateprofile();
 	void jabberupdateprofile();
 
-	void icqsynclist();
-	void yahoosynclist();
-
     public:
 	imcontroller();
 	~imcontroller();
 
 	void registration(icqconf::imaccount &account);
 	void updateinfo(icqconf::imaccount &account);
-	void synclist(icqconf::imaccount &account);
 };
 
 extern imcontroller imcontrol;
